import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()



class HEX(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[0-9]|[A-F]|[a-f]')


class EXPONENT(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[eE][+-]?[0-9]+')


class LANGTAG(jsg.JSGString):
    pattern = jsg.JSGPattern(r'@[a-zA-Z]+(\-[a-zA-Z0-9]+)*')

_CONTEXT.NAMESPACE = locals()



import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE = "id"
_CONTEXT.TYPE_EXCEPTIONS.append("person")
_CONTEXT.IGNORE.append("height")
_CONTEXT.IGNORE.append("weight")
_CONTEXT.IGNORE.append("location")


class person(jsg.JSGObject):
    _reference_types = []
    _members = {'name': jsg.String,
                'age': jsg.Integer}
    _strict = True

    def __init__(self,
                 name: str = None,
                 age: int = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.name = name
        self.age = age



class membership(jsg.JSGObject):
    _reference_types = []
    _members = {'list_name': jsg.String,
                'members': jsg.ArrayFactory('members', _CONTEXT, person, 0, None)}
    _strict = True

    def __init__(self,
                 list_name: str = None,
                 members: typing.List[person] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.list_name = list_name
        self.members = members


_CONTEXT.NAMESPACE = locals()



import os
import unittest


class GeneratePythonTestCase(unittest.TestCase):
    # Generate some bad python and verify that an error is detected in the output
    def test_execute_item(self):
        from pyjsg.parser_impl.generate_python import generate, evaluate

        basedir = os.path.abspath(os.path.dirname(__file__))
        goodfile = os.path.join(basedir, 'py', 'goodjsg.py')
        badfile = os.path.join(basedir, 'py', 'badjsg.py')

        # Make sure that a simple generate works
        self.assertEqual(0, generate([os.path.join(basedir, "jsg", "complexfacet.jsg"), "-o", goodfile, "-e", "-nh"]))

        # Make sure that we can detect exceptions and errors
        with open(badfile, 'w') as bf:
            bf.write("i=1/0\n")
            with open(goodfile) as gf:
                bf.write(gf.read())
        with self.assertRaises(ZeroDivisionError):
            evaluate("bar", badfile, False)

        # Make sure that our namespace hasn't been messed up
        self.assertFalse('bar' in globals())
        os.remove(goodfile)
        os.remove(badfile)


if __name__ == '__main__':
    unittest.main()



import os
import unittest
from contextlib import redirect_stdout
from io import StringIO
from typing import Optional, TextIO, cast

import requests
from dict_compare import compare_dicts
from jsonasobj import loads as jao_loads
from jsonasobj.jsonobj import as_dict, as_json

from pyjsg.validate_json import JSGPython

CWD = os.path.abspath(os.path.dirname(__file__))

# Test configuration variables
#     If you are going to be doing a lot of testing, it would be in your best interest to clone a copy of
#     https://github.com/shexSpec/shexTest into a local directory.  The configuration below is intended to
#     work with:
#     <path> +---  hsolbrig/pyjsg/tests/test_basics/test_shexj.py
#            |
#            +---- shexSpec/shexTest/schemas/*.jsg
#            |
#            +---- hsolbrig/ShExJSG/ShExJSG/ShExJ.jsg
#
# If you set LOCAL_TEST_FILES to True, shexTestRepository will be set as above.  You can obviously point it somewhere
# else if needed, but please don't check it in that way.
#
# USE_REL_TEST_PATHS = True attempts to hide absolute paths from logs and various things, but, as it makes debugging
# difficult, you can set it to False whilst debugging.
#
# If you set LOCAL_TEST_FILES to False, the following locations will be used:
#
#  - - - - -
#
# SHEXJ_JSG should point at the image of ShExJ.jsg to be used in the testing process.  We keep a local copy in
#    pyjsg/tests/test_basics/jsg/ShExJ.jsg to minimize the dependencies between ShExJSG and pyjsg.  One should
#    try to keep this up to date with the master, which can be found in:
#               https://github.com/hsolbrig/ShExJSG/ShExJSG/ShExJ.jsg
#
# ===== CHECKING IN =======
# LOCAL_TEST_FILES = False
# USE_REL_TEST_PATHS = True
# ONLY_TEST_THIS = ""
# STOP_ON_ERROR = False

LOCAL_TEST_FILES = False              # True means use files that have been checked out from repository, False repository
USE_REL_TEST_PATHS = True             # True means relative paths, false absolute
ONLY_TEST_THIS = ""                   # If not empty, path to a single file you want to do a test on
STOP_ON_ERROR = False                 # True means stop on first error.  False means test all jsg

if LOCAL_TEST_FILES:
    # Files are local and stored relative to this directory
    organization_root = os.path.join(CWD, '../../../')            # test_basic/tests/pyjsg/  HERE
    shexJSGSource = os.path.relpath(os.path.join(organization_root, 'ShExJSG/ShExJSG/ShExJ.jsg'), CWD)
    shexTestRepository = os.path.relpath(os.path.join(organization_root, '../shexSpec/shexTest/schemas'), CWD)
    if not USE_REL_TEST_PATHS:
        shexJSGSource = os.path.abspath(shexJSGSource)
        shexTestRepository = os.path.abspath(shexTestRepository)
    assert os.path.exists(shexJSGSource)
    assert os.path.exists(shexTestRepository)
else:
    shexTestRepository = "https://api.github.com/repos/shexSpec/shexTest/contents/schemas?ref=main"
    shexJSGSource = "https://raw.githubusercontent.com/hsolbrig/ShExJSG/master/ShExJSG/ShExJ.jsg"
print(f"TESTING *.json from: {shexTestRepository} against {shexJSGSource}")


# Files to skip until we reintroduce a manifest reader
skip = ['coverage.json', 'manifest.json', 'representationTests.json']


def compare_json(j1: str, j2: str, log: TextIO) -> bool:
    """ Compare two JSON strings """
    d1 = jao_loads(j1)
    d2 = jao_loads(j2)
    return compare_dicts(as_dict(d1), as_dict(d2), file=log)


def validate_shexj_json(json_str: str, input_fname: str, parser: JSGPython) -> bool:
    """Validate json_str against ShEx Schema

    :param json_str: String to validate
    :param input_fname: Name of source file for error reporting
    :param parser: JSGPython parser
    :return: True if pass
    """
    rslt = parser.conforms(json_str, input_fname)
    if not rslt.success:
        print("File: {} - ".format(input_fname))
        print(str(rslt.fail_reason))
        return False
    else:
        log = StringIO()
        if not compare_json(json_str, as_json(parser.json_obj), cast(TextIO, log)):
            print("File: {} - ".format(input_fname))
            print(log.getvalue())
            print(as_json(parser.json_obj))
            return False
    return True


class FileValidator:
    def __init__(self):
        self.nvalidated = 0
        self.nskipped = 0

    def validate_file(self, download_file: str, parser: JSGPython) -> bool:
        fname = download_file.rsplit('/', 1)[1]
        if fname not in skip and 'futureWork' not in download_file:
            if '://' in download_file:
                resp = requests.get(download_file)
                if resp.ok:
                    text = resp.text
                else:
                    print("Error {}: {}".format(resp.status_code, resp.reason))
                    return False
            else:
                with open(download_file) as f:
                    text = f.read()
            self.nvalidated += 1
            return validate_shexj_json(text, download_file, parser)
        print("Skipping {}".format(download_file))
        self.nskipped += 1
        return True


def download_github_file(github_url: str) -> Optional[str]:
    """
    Download the file in github_url
    :param github_url: github url to download
    :return: file contents if success, None otherwise
    """
    print("Downloading {}".format(github_url))
    resp = requests.get(github_url)
    if resp.ok:
        resp = requests.get(resp.json()['download_url'])
        if resp.ok:
            return resp.text
    print("Error {}: {}".format(resp.status_code, resp.reason))
    return None


def validate_shex_schemas(parser: JSGPython, validator: FileValidator) -> bool:
    if not ONLY_TEST_THIS:
        filelist = []
        if '://' in shexTestRepository:
            resp = requests.get(shexTestRepository)
            if resp.ok:
                filelist = [f['download_url'] for f in resp.json() if f['name'].endswith('.json')]
            else:
                print("Error {}: {}".format(resp.status_code, resp.reason))
                return False
        else:
            for dirpath, _, filenames in os.walk(shexTestRepository):
                filelist += [os.path.join(dirpath, f) for f in filenames if f.endswith('.json')]
        if STOP_ON_ERROR:
            return all(validator.validate_file(f, parser) for f in filelist)
        else:
            return all([validator.validate_file(f, parser) for f in filelist])
    else:
        return validator.validate_file(ONLY_TEST_THIS, parser)


class ShExJValidationTestCase(unittest.TestCase):

    @unittest.skipIf(False, "Schema tests disabled -- do not commit in this state")
    def test_shex_schema(self):
        parser = JSGPython(shexJSGSource)
        log_path = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'logs', 'test_shex_schema.log')
        validator = FileValidator()
        with open(log_path, 'w') as logf:
            with redirect_stdout(logf):
                self.assertTrue(validate_shex_schemas(parser, validator), f"See {log_path} for reasons")
        print(f"{validator.nvalidated} files validated - {validator.nskipped} skipped")


if __name__ == '__main__':
    unittest.main()



import sys
import unittest
from typing import Union

import os

from pyjsg.jsglib import isinstance_


class IsInstanceTestCase(unittest.TestCase):
    def test_isinstance_issue(self):
        from pyjsg.jsglib.loader import isinstance_
        x = Union[int, str]
        # Typing library seems to support Unions as of python 3.10
        if sys.version_info < (3, 10):
            with self.assertRaises(TypeError):
                isinstance(17, x)
        else:
            self.assertTrue(isinstance(17, x))
        self.assertTrue(isinstance_(17, x))

    def test_issue_with_shexj(self):
        from pyjsg.parser_impl.generate_python import generate
        data_root = os.path.join(os.path.split(os.path.abspath(__file__))[0], '..', 'test_basics')
        jsg_path = os.path.relpath(os.path.join(data_root, 'jsg', 'ShExJ.jsg'))
        py_path = os.path.abspath(os.path.join(data_root, 'py', 'ShExJ.py'))
        self.assertEqual(0, generate([jsg_path, "-o", py_path, "-e", "-nh"]))
        from tests.test_basics.py import ShExJ
        self.assertTrue(isinstance_(ShExJ.IRIREF("http://foo.bar"), ShExJ.shapeExprLabel))

if __name__ == '__main__':
    unittest.main()



# Auto generated from JSGPython by PyJSG version 0.8b1
# Generation date: 2018-07-31 11:27
#
import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("doc")


class doc(jsg.JSGObject):
    _reference_types = []
    _members = {'l23 38n': jsg.ArrayFactory('l23 38n', _CONTEXT, jsg.ArrayFactory('l23 38n', _CONTEXT, jsg.String, 2, 8), 2, 3)}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        setattr(self, 'l23 38n', _kwargs.get('l23 38n', None))


_CONTEXT.NAMESPACE = locals()



import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("labeledShapeOr")
_CONTEXT.TYPE_EXCEPTIONS.append("labeledShapeAnd")
_CONTEXT.TYPE_EXCEPTIONS.append("labeledShapeNot")
_CONTEXT.TYPE_EXCEPTIONS.append("labeledNodeConstraint")
_CONTEXT.TYPE_EXCEPTIONS.append("labeledShape")
_CONTEXT.TYPE_EXCEPTIONS.append("shapeExprLabel")
_CONTEXT.TYPE_EXCEPTIONS.append("labeledShapeExternal")
_CONTEXT.TYPE_EXCEPTIONS.append("expr_1_")
_CONTEXT.TYPE_EXCEPTIONS.append("expr_2_")
_CONTEXT.TYPE_EXCEPTIONS.append("expr_3_")
_CONTEXT.TYPE_EXCEPTIONS.append("expr_4_")
_CONTEXT.TYPE_EXCEPTIONS.append("expr_5_")
_CONTEXT.TYPE_EXCEPTIONS.append("expr_6_")
_CONTEXT.TYPE_EXCEPTIONS.append("expr_7_")
_CONTEXT.TYPE_EXCEPTIONS.append("expr")


class labeledShapeOr(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class labeledShapeAnd(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class labeledShapeNot(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class labeledNodeConstraint(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class labeledShape(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class shapeExprLabel(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class labeledShapeExternal(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class expr_1_(jsg.JSGObject):
    _reference_types = [labeledShapeOr]
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class expr_2_(jsg.JSGObject):
    _reference_types = [labeledShapeAnd]
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class expr_3_(jsg.JSGObject):
    _reference_types = [labeledShapeNot]
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class expr_4_(jsg.JSGObject):
    _reference_types = [labeledNodeConstraint]
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class expr_5_(jsg.JSGObject):
    _reference_types = [labeledShape]
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class expr_6_(jsg.JSGObject):
    _reference_types = [shapeExprLabel]
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class expr_7_(jsg.JSGObject):
    _reference_types = [labeledShapeExternal]
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class expr(jsg.JSGObject):
    _reference_types = [expr_1_, expr_2_, expr_3_, expr_4_, expr_5_, expr_6_, expr_7_]
    _members = {}
    _strict = True

    def __init__(self,
                 opts_: typing.Union[expr_1_, expr_2_, expr_3_, expr_4_, expr_5_, expr_6_, expr_7_] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        if opts_ is not None:
            if isinstance(opts_, expr_1_):
                pass
            elif isinstance(opts_, expr_2_):
                pass
            elif isinstance(opts_, expr_3_):
                pass
            elif isinstance(opts_, expr_4_):
                pass
            elif isinstance(opts_, expr_5_):
                pass
            elif isinstance(opts_, expr_6_):
                pass
            elif isinstance(opts_, expr_7_):
                pass
            else:
                raise ValueError(f"Unrecognized value type: {opts_}")


_CONTEXT.NAMESPACE = locals()



import unittest

from tests.utils.harness import Harness


class MissingItemTestCase(Harness):
    def test1(self):
        # If status is misssing completely, we fail
        # Args: jsg, json, should_pass, raises_exception, fails_validation
        # Fails because of required status
        self.do_test("doc {status: .}", "{}", False, False, True, "doc: Missing required field: 'status'")
        # Passes because item matches
        self.do_test("doc {status: .} item {}", "{}")
        # Fails because neither status nor v present
        self.do_test("doc {status: .} item {v: .}", "{}", False, False, True, "doc: Missing required field: 'status'")
        self.do_test("item {v: .} doc {status: .} ", "{}", False, False, True, "item: Missing required field: 'v'")
        # If status is present in ANY form, it passes
        self.do_test("doc {status: .} item {v: .}", '{"status": null}')
        self.do_test("doc {status: .} item {v: .}", '{"status": 17}')
        self.do_test("doc {status: .} item {v: .}", '{"status": "stuff"}')
        # If v is present in any for it passes as well
        self.do_test("doc {status: .} item {v: .}", '{"v": true}', True)


if __name__ == '__main__':
    unittest.main()



# Auto generated from JSGPython by PyJSG version 0.8b1
# Generation date: 2018-07-31 11:31
#
import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("doc")
_CONTEXT.TYPE_EXCEPTIONS.append("another_object")


class doc(jsg.JSGObject):
    _reference_types = []
    _members = {'v1': jsg.String,
                'v2': jsg.Number,
                'v3': jsg.Integer,
                'v4': jsg.Boolean,
                'v5': jsg.JSGNull,
                'v6': jsg.ArrayFactory('v6', _CONTEXT, jsg.AnyType, 0, None),
                'v7': jsg.ObjectFactory('v7', _CONTEXT, jsg.Object)}
    _strict = True

    def __init__(self,
                 v1: str = None,
                 v2: float = None,
                 v3: int = None,
                 v4: bool = None,
                 v5: type(None) = jsg.Empty,
                 v6: list = None,
                 v7: object = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.v1 = v1
        self.v2 = v2
        self.v3 = v3
        self.v4 = v4
        self.v5 = v5
        self.v6 = v6
        self.v7 = v7



class another_object(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = False

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)


_CONTEXT.NAMESPACE = locals()




import unittest
from typing import Any

from pyjsg.jsglib import JSGString, JSGPattern
from tests.test_jsglib.iri_defn import *


class JSGStringTestCase(unittest.TestCase):
    def _do_test(self, jsgstr, val: Any, is_valid: bool=True) -> None:
        if not is_valid:
            with self.assertRaises(ValueError):
                jsgstr(val)
        else:
            v = jsgstr(val)
            self.assertEqual(v, str(val))

    def test_wildcard(self):
        class WildCard(JSGString):
            pattern = JSGPattern(r'.*')
            python_type = object

        self._do_test(WildCard, "")
        self._do_test(WildCard, False)
        self._do_test(WildCard, "any block of text")
        self._do_test(WildCard, "^<80>߿ࠀ࿿က쿿퀀퟿�𐀀𿿽񀀀󿿽􀀀􏿽$/")
        self._do_test(WildCard, "\n\uFDF0*")

    def test_fixedvalues(self):
        class IRI(JSGString):
            pattern = JSGPattern(r'http:\/\/www\.w3\.org\/ns\/shex\.jsonld')
            
        self._do_test(IRI,"http://www.w3.org/ns/shex.jsonld")
        self._do_test(IRI,"http://www.w3.org/ns/shex/jsonldx", False)
        self._do_test(IRI,"http://www.w3.org/ns/shex/jsonld ", False)
        self._do_test(IRI,"http://www.w3.org/ns/shex/jsonld\n", False)
        self._do_test(IRI," http://www.w3.org/ns/shex/jsonld", False)

    def test_alternatives(self):
        class Alts(JSGString):
            pattern = JSGPattern(r'iri|bnode|nonliteral|literal')
        self._do_test(Alts,'iri')
        self._do_test(Alts,'literal')
        self._do_test(Alts,'nonliteral')
        self._do_test(Alts,'bnode')
        self._do_test(Alts,'node', False)
        self._do_test(Alts,'bnod', False)
        self._do_test(Alts,'IRI', False)
        self._do_test(Alts,' iri', False)
        self._do_test(Alts,'iri ', False)

    def test_bool(self):
        class BOOL(JSGString):
            pattern = JSGPattern(r'[Tt]rue|[Ff]alse')
            python_type = (str, bool)

        self._do_test(BOOL, 'true')
        self._do_test(BOOL, 'false')
        self._do_test(BOOL, 'TRUE', False)
        self._do_test(BOOL, 'True')
        self._do_test(BOOL, 'False')
        self._do_test(BOOL, True)
        self._do_test(BOOL, False)
        self._do_test(BOOL, 0, False)
        self._do_test(BOOL, None, False)

    def test_assorted(self):
        class INT(JSGString):
            pattern = JSGPattern(r'[+-]?[0-9]+')
            
        self._do_test(INT, "0")
        self._do_test(INT, str(-173))
        self._do_test(INT, "+11720000845197308888890")
        self._do_test(INT, "01")
        self._do_test(INT, "--17", False)
        self._do_test(INT, "1.0", False)
    
        class NUM(JSGString):
            pattern = JSGPattern(r'[+-]?[0-9]*\.[0-9]+')
            
        self._do_test(NUM, "0", False)
        self._do_test(NUM, "0.0")
        self._do_test(NUM, str(float(-173)))
        self._do_test(NUM, "1.0")
        self._do_test(NUM, "+11720000845197308888.0000000000")

        self._do_test(IRI, "http://a.example/p\u0031")

if __name__ == '__main__':
    unittest.main()



import sys
import unittest
from tokenize import Double
from typing import cast

from jsonasobj import JsonObj

import pyjsg.jsglib as jsg
import pyjsg.jsglib.jsg_null
from pyjsg.parser_impl.jsg_lexerruleblock_parser import JSGLexerRuleBlock
from tests.test_basics.parser import parse

# Note: Python tightened up the re.escape() functionality in version 3.7 -- it had previously been fairly promiscuous
# when deciding what to escape.  The escape call itself can be found in jsg_lexerruleblock_parser.py.add_string().
# This is why the conditionals below

# Force the needed imports
_x = isinstance(1, (jsg.JSGPattern, jsg.JSGString, jsg.String, jsg.Number, Double, jsg.Boolean, jsg.Integer,
                    pyjsg.jsglib.jsg_null.JSGNull, jsg.JSGArray, JsonObj))


# Required to get the lexer in the correct state
terminals = """@terminals
"""


t1 = "IRI : (PN_CHARS | '.' | ':' | '/' | '\\' | '#' | '@' | '%' | '&' | UCHAR)* ; " \
     "# <http://www.w3.org/TR/turtle/#grammar-production-IRIREF> "
t2 = "BNODE : '_:' (PN_CHARS_U | [0-9]) ((PN_CHARS | '.')* PN_CHARS)? ; " \
     "# <http://www.w3.org/TR/turtle/#grammar-production-BLANK_NODE_LABEL>"
t3 = 'BOOL : "true" | "false" ; # JSON boolean tokens'
t4 = 'INTEGER : [+-]? [0-9] + ; # <http://www.w3.org/TR/turtle/#grammar-production-INTEGER>'
t5 = "DECIMAL : [+-]? [0-9]* '.' [0-9] + ; # <http://www.w3.org/TR/turtle/#grammar-production-DECIMAL>"
t6 = "DOUBLE : [+-]? ([0-9] + '.' [0-9]* EXPONENT | '.' [0-9]+ EXPONENT | [0-9]+ EXPONENT) ; " \
     "# <http://www.w3.org/TR/turtle/#grammar-production-DOUBLE>"
t7 = 'STRING : .* ;'

t8 = "PN_PREFIX  : PN_CHARS_BASE ((PN_CHARS | '.')* PN_CHARS)? ;"
t9 = """PN_CHARS_BASE    : [A-Z] | [a-z] | [\u00C0-\u00D6] | [\u00D8-\u00F6]
                 | [\u00F8-\u02FF] | [\u0370-\u037D] | [\u037F-\u1FFF]
                 | [\u200C-\u200D] | [\u2070-\u218F] | [\u2C00-\u2FEF]
                 | [\u3001-\uD7FF] | [\uF900-\uFDCF] | [\uFDF0-\uFFFD]
                 | [\u10000-\uEFFFF] ;"""
t10 = "PN_CHARS : PN_CHARS_U | '-' | [0-9] | '\u00B7' | [\u0300-\u036F] | [\u203F-\u2040] ;"
t11 = "PN_CHARS_U       : PN_CHARS_BASE | '_' ;"
t12 = """UCHAR            : '\\u' HEX HEX HEX HEX
                 | '\\U' HEX HEX HEX HEX HEX HEX HEX HEX ; """
t13 = "HEX : [0-9] | [A-F] | [a-f] ; "
t14 = "EXPONENT : [eE] [+-]? [0-9]+ ; "
t15 = "LANGTAG          : '@' [a-zA-Z] + ('-' [a-zA-Z0-9] +)* ;"
t16 = "STR_: .* @string ;"
t17 = "NUMBER_ : .* @number ;"
t18 = "INT_: .* @int ;"
t19 = "BOOL_ : .* @bool ;"
t20 = "NULL_: .* @null ;"
# TODO: Figure out how these even parsed
# t21 = "ARRAY_ : .* @array ;"
# t22 = "OBJECT_: .* @object ;"
t23 = "POS_INT : [0]|([1-9][0-9]*) @int ;"

if sys.version_info < (3, 7):
    s1 = "pattern: r'(({PN_CHARS})|\.|\:|\/|\\\\|\#|\@|\%|\&|({UCHAR}))*'"
    s2 = "pattern: r'_\:(({PN_CHARS_U})|[0-9])((({PN_CHARS})|\.)*({PN_CHARS}))?'"
else:
    s1 = "pattern: r'(({PN_CHARS})|\.|:|/|\\\\|\#|@|%|\&|({UCHAR}))*'"
    s2 = "pattern: r'_:(({PN_CHARS_U})|[0-9])((({PN_CHARS})|\.)*({PN_CHARS}))?'"
s3 = "pattern: r'true|false'"
s4 = "pattern: r'[+-]?[0-9]+'"
s5 = "pattern: r'[+-]?[0-9]*\.[0-9]+'"
s6 = "pattern: r'[+-]?([0-9]+\.[0-9]*({EXPONENT})|\.[0-9]+({EXPONENT})|[0-9]+({EXPONENT}))'"
s7 = "pattern: r'.*'"
s8 = "pattern: r'({PN_CHARS_BASE})((({PN_CHARS})|\.)*({PN_CHARS}))?'"
s9 = "pattern: r'[A-Z]|[a-z]|[À-Ö]|[Ø-ö]|[ø-˿]|[Ͱ-ͽ]|[Ϳ-῿]|[‌-‍]|[⁰-↏]|[Ⰰ-⿯]|[、-퟿]|[豈-﷏]|[ﷰ-�]|[က0-F]'"
if sys.version_info < (3, 7):
    s10 = "pattern: r'({PN_CHARS_U})|\-|[0-9]|\·|[̀-ͯ]|[‿-⁀]'"
else:
    s10 = "pattern: r'({PN_CHARS_U})|\-|[0-9]|·|[̀-ͯ]|[‿-⁀]'"
s11 = "pattern: r'({PN_CHARS_BASE})|_'"
s12 = "pattern: r'\\\\u({HEX})({HEX})({HEX})({HEX})|\\\\U({HEX})({HEX})({HEX})({HEX})({HEX})({HEX})({HEX})({HEX})'"
s13 = "pattern: r'[0-9]|[A-F]|[a-f]'"
s14 = "pattern: r'[eE][+-]?[0-9]+'"
if sys.version_info < (3, 7):
    s15 = "pattern: r'\@[a-zA-Z]+(\-[a-zA-Z0-9]+)*'"
else:
    s15 = "pattern: r'@[a-zA-Z]+(\-[a-zA-Z0-9]+)*'"
s16 = "pattern: r'.*'"
s17 = "pattern: r'.*'"
s18 = "pattern: r'.*'"
s19 = "pattern: r'.*'"
s20 = "pattern: r'.*'"
s21 = "pattern: r'.*'"
s22 = "pattern: r'.*'"
s23 = "pattern: r'[0]|([1-9][0-9]*)'"


if sys.version_info < (3, 7):
    r1 = r'''class IRI(jsg.JSGString):
    pattern = jsg.JSGPattern(r'(({PN_CHARS})|\.|\:|\/|\\|\#|\@|\%|\&|({UCHAR}))*'.format(PN_CHARS=PN_CHARS.pattern, UCHAR=UCHAR.pattern))'''
    r2 = '''class BNODE(jsg.JSGString):
    pattern = jsg.JSGPattern(r'_\:(({PN_CHARS_U})|[0-9])((({PN_CHARS})|\.)*({PN_CHARS}))?'.format(PN_CHARS=PN_CHARS.pattern, PN_CHARS_U=PN_CHARS_U.pattern))'''
else:
    r1 = r'''class IRI(jsg.JSGString):
    pattern = jsg.JSGPattern(r'(({PN_CHARS})|\.|:|/|\\|\#|@|%|\&|({UCHAR}))*'.format(PN_CHARS=PN_CHARS.pattern, UCHAR=UCHAR.pattern))'''
    r2 = '''class BNODE(jsg.JSGString):
    pattern = jsg.JSGPattern(r'_:(({PN_CHARS_U})|[0-9])((({PN_CHARS})|\.)*({PN_CHARS}))?'.format(PN_CHARS=PN_CHARS.pattern, PN_CHARS_U=PN_CHARS_U.pattern))'''

r3 = '''class BOOL(jsg.JSGString):
    pattern = jsg.JSGPattern(r'true|false')'''
r4 = '''class INTEGER(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[+-]?[0-9]+')'''
r5 = '''class DECIMAL(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[+-]?[0-9]*\.[0-9]+')'''
r6 = '''class DOUBLE(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[+-]?([0-9]+\.[0-9]*({EXPONENT})|\.[0-9]+({EXPONENT})|[0-9]+({EXPONENT}))'.format(EXPONENT=EXPONENT.pattern))'''
r7 = '''class STRING(jsg.JSGString):
    pattern = jsg.JSGPattern(r'.*')'''
r8 = '''class PN_PREFIX(jsg.JSGString):
    pattern = jsg.JSGPattern(r'({PN_CHARS_BASE})((({PN_CHARS})|\.)*({PN_CHARS}))?'.format(PN_CHARS=PN_CHARS.pattern, PN_CHARS_BASE=PN_CHARS_BASE.pattern))'''
r9 = '''class PN_CHARS_BASE(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[A-Z]|[a-z]|[À-Ö]|[Ø-ö]|[ø-˿]|[Ͱ-ͽ]|[Ϳ-῿]|[‌-‍]|[⁰-↏]|[Ⰰ-⿯]|[、-퟿]|[豈-﷏]|[ﷰ-�]|[က0-F]')'''
if sys.version_info < (3, 7):
    r10 = '''class PN_CHARS(jsg.JSGString):
    pattern = jsg.JSGPattern(r'({PN_CHARS_U})|\-|[0-9]|\·|[̀-ͯ]|[‿-⁀]'.format(PN_CHARS_U=PN_CHARS_U.pattern))'''
else:
    r10 = '''class PN_CHARS(jsg.JSGString):
    pattern = jsg.JSGPattern(r'({PN_CHARS_U})|\-|[0-9]|·|[̀-ͯ]|[‿-⁀]'.format(PN_CHARS_U=PN_CHARS_U.pattern))'''
r11 = '''class PN_CHARS_U(jsg.JSGString):
    pattern = jsg.JSGPattern(r'({PN_CHARS_BASE})|_'.format(PN_CHARS_BASE=PN_CHARS_BASE.pattern))'''
r12 = '''class UCHAR(jsg.JSGString):
    pattern = jsg.JSGPattern(r'\\\\u({HEX})({HEX})({HEX})({HEX})|\\\\U({HEX})({HEX})({HEX})({HEX})({HEX})({HEX})({HEX})({HEX})'.format(HEX=HEX.pattern))'''
r13 = '''class HEX(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[0-9]|[A-F]|[a-f]')'''
r14 = '''class EXPONENT(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[eE][+-]?[0-9]+')'''
if sys.version_info < (3, 7):
    r15 = '''class LANGTAG(jsg.JSGString):
    pattern = jsg.JSGPattern(r'\@[a-zA-Z]+(\-[a-zA-Z0-9]+)*')'''
else:
    r15 = '''class LANGTAG(jsg.JSGString):
    pattern = jsg.JSGPattern(r'@[a-zA-Z]+(\-[a-zA-Z0-9]+)*')'''
r16 = '''class STR_(jsg.String):
    pattern = jsg.JSGPattern(r'.*')'''
r17 = '''class NUMBER_(jsg.Number):
    pattern = jsg.JSGPattern(r'.*')'''
r18 = '''class INT_(jsg.Integer):
    pattern = jsg.JSGPattern(r'.*')'''
r19 = '''class BOOL_(jsg.Boolean):
    pattern = jsg.JSGPattern(r'.*')'''
r20 = '''class NULL_(jsg.JSGNull):
    pattern = jsg.JSGPattern(r'.*')'''
r21 = '''class ARRAY_(Array):
    pattern = jsg.JSGPattern(r'.*')'''
r22 = '''class OBJECT_(JsonObj):
    pattern = jsg.JSGPattern(r'.*')'''
r23 = '''class POS_INT(jsg.Integer):
    pattern = jsg.JSGPattern(r'[0]|([1-9][0-9]*)')'''


exec(r13)
exec(r12)
exec(r9)
exec(r11)
exec(r10)
exec(r14)


tests = [(t1, 'IRI', r1, s1), (t2, 'BNODE', r2, s2), (t3, 'BOOL', r3, s3), (t4, 'INTEGER', r4, s4),
         (t5, 'DECIMAL', r5, s5), (t6, 'DOUBLE', r6, s6), (t7, 'STRING', r7, s7), (t8, 'PN_PREFIX', r8, s8),
         (t9, 'PN_CHARS_BASE', r9, s9), (t10, 'PN_CHARS', r10, s10), (t11, 'PN_CHARS_U', r11, s11),
         (t12, 'UCHAR', r12, s12), (t13, 'HEX', r13, s13), (t14, 'EXPONENT', r14, s14), (t15, 'LANGTAG', r15, s15),
         (t16, 'STR_', r16, s16), (t17, 'NUMBER_', r17, s17), (t18, 'INT_', r18, s18),
         (t19, 'BOOL_', r19, s19), (t20, 'NULL_', r20, s20),
         # (t21, 'ARRAY_', r21, s21), (t22, 'OBJECT_', r22, s22),
         (t23, 'POS_INT', r23, s23)]


class LexerRuleBlockTestCase(unittest.TestCase):
    def test1(self):
        i = 0
        for text, k, rslt, s in tests:
            i += 1
            t = cast(JSGLexerRuleBlock, parse(terminals + text, "lexerRules", JSGLexerRuleBlock))
            self.assertEqual(s, str(t))
            t_python = t.as_python(k)
            self.assertEqual(rslt, t_python.strip())
            exec(t_python, globals(), locals())


if __name__ == '__main__':
    unittest.main()



import unittest
from typing import cast

from pyjsg.parser_impl.jsg_builtinvaluetype_parser import JSGBuiltinValueType
from pyjsg.parser_impl.jsg_valuetype_parser import JSGValueType
from tests.test_basics.parser import parse

builtin_tests = [("@string", "jsg.String", "str", "None"),
                 ("@number", "jsg.Number", "float", "None"),
                 ("@int", "jsg.Integer", "int", "None"),
                 ("@bool", "jsg.Boolean", "bool", "None"),
                 ("@null", "jsg.JSGNull", "type(None)", "jsg.Empty"),
                 ("@array", "jsg.ArrayFactory('{name}', _CONTEXT, jsg.AnyType, 0, None)", "list", "None"),
                 ("@object", "jsg.ObjectFactory('{name}', _CONTEXT, jsg.Object)", "object", "None"),
                 (".", "jsg.AnyTypeFactory('{name}', _CONTEXT)", "object", "jsg.Empty")]


class BuiltinValueTypeTestCase(unittest.TestCase):
    def test_builtins(self):
        for text, sig, typ_, mt_typ in builtin_tests:
            t = cast(JSGValueType, parse(text, "builtinValueType", JSGBuiltinValueType))
            self.assertEqual(sig, t.signature_type(), text)
            self.assertEqual(typ_, t.python_type(), text)
            self.assertEqual(f"builtinValueType: {text if text != '.' else 'jsg.AnyType'}", str(t), text)
            self.assertEqual(mt_typ, t.mt_value(), text)
            self.assertEqual([], t.members_entries(), text)
            self.assertEqual([], t.dependency_list(), text)


if __name__ == '__main__':
    unittest.main()




import unittest
from typing import cast

from pyjsg.parser_impl.jsg_doc_parser import JSGDocParser
from tests.test_basics.parser import parse


class ObjectMacroTestCase(unittest.TestCase):
    def test_single_option(self):
        t = cast(JSGDocParser, parse("macro = labeledShapeOr ; labeledShapeOr {}", "doc", JSGDocParser))
        exec(t.as_python(self.__class__.__name__), dict())
        t = cast(JSGDocParser, parse("macro = a:@int | b:@int ;", "doc", JSGDocParser))
        exec(t.as_python(self.__class__.__name__), dict())
        t = cast(JSGDocParser, parse("macro = a| b ; a {} b {}", "doc", JSGDocParser))
        exec(t.as_python(self.__class__.__name__), dict())


if __name__ == '__main__':
    unittest.main()



# Auto generated from JSGPython by PyJSG version 0.7b2
# Generation date: 2018-07-18 18:06
#
import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("Shape")


class Shape(jsg.JSGObject):
    _reference_types = []
    _members = {'id': typing.Optional[typing.Union[jsg.Integer, jsg.String]]}
    _strict = True

    def __init__(self,
                 id: typing.Optional[typing.Union[int, str]] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.id = id


_CONTEXT.NAMESPACE = locals()



import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("doc")


class NAME(jsg.JSGString):
    pattern = jsg.JSGPattern(r'.*')


class TEMPLATE(jsg.JSGString):
    pattern = jsg.JSGPattern(r'\{.*\}')


nameOrTemplate = typing.Union[NAME, TEMPLATE]


class doc(jsg.JSGObject):
    _reference_types = []
    _members = {'street': nameOrTemplate}
    _strict = True

    def __init__(self,
                 street: typing.Union[str, str] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.street = street


_CONTEXT.NAMESPACE = locals()



import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("list_eval")


class INT(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[0-9]')


class list_eval(jsg.JSGObject):
    _reference_types = []
    _members = {'req': INT,
                'opt': typing.Optional[INT],
                'zero_or_more': jsg.ArrayFactory('zero_or_more', _CONTEXT, INT, 0, None),
                'one_or_more': jsg.ArrayFactory('one_or_more', _CONTEXT, INT, 1, None),
                'two_or_more': jsg.ArrayFactory('two_or_more', _CONTEXT, INT, 2, None),
                'three_or_four': jsg.ArrayFactory('three_or_four', _CONTEXT, INT, 3, 4),
                'one_or_more_v2': jsg.ArrayFactory('one_or_more_v2', _CONTEXT, INT, 1, None)}
    _strict = True

    def __init__(self,
                 req: str = None,
                 opt: typing.Optional[str] = None,
                 zero_or_more: typing.List[str] = None,
                 one_or_more: typing.List[str] = None,
                 two_or_more: typing.List[str] = None,
                 three_or_four: typing.List[str] = None,
                 one_or_more_v2: typing.List[str] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.req = req
        self.opt = opt
        self.zero_or_more = zero_or_more
        self.one_or_more = one_or_more
        self.two_or_more = two_or_more
        self.three_or_four = three_or_four
        self.one_or_more_v2 = one_or_more_v2


_CONTEXT.NAMESPACE = locals()



import os
from argparse import ArgumentParser
from io import StringIO
from types import ModuleType
from typing import cast, TextIO, NamedTuple, Optional

import requests

from pyjsg.jsglib.loader import loads, Logger, is_valid
from pyjsg.parser_impl.generate_python import parse


class ValidationResult(NamedTuple):
    success: bool
    fail_reason: str
    test_name: str
    type: Optional[str]

    def __str__(self) -> str:
        return (f"{self.test_name}: " if self.test_name else "") +\
               (f"Conforms to {self.type}" if self.success else f"FAIL - {self.fail_reason}")


class JSGPython:
    def __init__(self, jsg: Optional[str]=None, python: Optional[str]=None, print_python: bool=False) -> None:
        """ Construct a jsg validation module

        :param jsg: JSG specification.  If none, use python
        :param python: Python specification.
        :param print_python: True means print Python to stdout
        """
        if jsg is not None:
            self.schema = self._to_string(jsg) if not self._is_jsg(jsg) else jsg
        else:
            self.schema = None
        self.python = parse(self.schema, self.__class__.__name__) if self.schema else self._to_string(python)
        if print_python:
            print(self.python)
        self.json_obj = None
        if not self.python:
            raise ValueError("JSGPython: jsg parsing error")
        spec = compile(self.python, self.__class__.__name__, 'exec')
        self.module = ModuleType(self.__class__.__name__)
        exec(spec, self.module.__dict__)

    @staticmethod
    def _is_jsg(s: str) -> bool:
        """ Determine whether s looks like a JSG spec """
        return isinstance(s, str) and ('\n' in s or '{' in s)

    @staticmethod
    def is_json(s: str) -> bool:
        """ Determine whether s looks like JSON """
        return s.strip().startswith(('{', '['))

    @staticmethod
    def _to_string(inp: str) -> str:
        """ Convert a URL or file name to a string """
        if '://' in inp:
            req = requests.get(inp)
            if not req.ok:
                raise ValueError(f"Unable to read {inp}")
            return req.text
        else:
            with open(inp) as infile:
                return infile.read()

    def conforms(self, json: str, name: str = "", verbose: bool=False) -> ValidationResult:
        """ Determine whether json conforms with the JSG specification

        :param json: JSON string, URI to JSON or file name with JSON
        :param name: Test name for ValidationResult -- printed in dx if present
        :param verbose: True means print the response
        :return: pass/fail + fail reason
        """
        json = self._to_string(json) if not self.is_json(json) else json
        try:
            self.json_obj = loads(json, self.module)
        except ValueError as v:
            return ValidationResult(False, str(v), name, None)
        logfile = StringIO()
        logger = Logger(cast(TextIO, logfile))      # cast because of bug in ide
        if not is_valid(self.json_obj, logger):
            return ValidationResult(False, logfile.getvalue().strip('\n'), name, None)
        return ValidationResult(True, "", name, type(self.json_obj).__name__)


def genargs() -> ArgumentParser:
    """
    Create a command line parser

    :return: parser
    """
    parser = ArgumentParser()
    parser.add_argument("spec", help="JSG specification - can be file name, URI or string")
    parser.add_argument("-o", "--outfile", help="Output python file - if omitted, python is not saved")
    parser.add_argument("-p", "--print", help="Print python file to stdout")
    parser.add_argument("-id", "--inputdir", help="Input directory with JSON files")
    parser.add_argument("-i", "--json", help="URL, file name or json text", nargs='*')
    return parser


def validate_json(argv) -> bool:
    def do_validation(entry: str) -> bool:
        if opts.verbose and not validator.is_json(entry):
            print(f"Validating {entry}... ", end='')
        success, reason = validator.conforms(entry)
        if opts.verbose:
            if success:
                print("Success")
                return True
            else:
                print(f"Fail: {reason}")
                return False

    opts = genargs().parse_args(argv)
    validator = JSGPython(opts.spec, print_python=opts.print)
    all_pass = True

    if opts.outfile:
        with open(opts.outfile, 'w') as outf:
            outf.write(validator.python)

    if opts.indir:
        for filedir, _, files in os.walk(opts.indir):
            for file in files:
                if file.endswith('.json') or file.endswith('.jsonld'):
                    fname = os.path.join(filedir, file)
                    if not do_validation(fname):
                        all_pass = False

    if opts.json:
        for json in opts.json:
            if not do_validation(json):
                all_pass = False
    return all_pass




import unittest
from typing import cast

from pyjsg.parser_impl.jsg_arrayexpr_parser import JSGArrayExpr
from tests.test_basics.parser import parse


class ArrayExprTestCase(unittest.TestCase):
    def check(self, t: JSGArrayExpr, expected) -> None:
        tup = (str(t), t.python_type(), t.dependency_list(), t.signature_type(), t.mt_value(), t.members_entries())
        self.assertEqual(expected, tup)

    def test_basics(self):
        t = cast(JSGArrayExpr, parse("id [@string] ", "arrayDef", JSGArrayExpr))
        self.check(t,
              ('arrayExpr: [valueType: builtinValueType: @string]',
               'typing.List[str]',
                [],
                "jsg.ArrayFactory('{name}', _CONTEXT, jsg.String, 0, None)",
                'None',
                []))

        t = cast(JSGArrayExpr, parse("id [@int] ", "arrayDef", JSGArrayExpr))
        self.check(t,
              ('arrayExpr: [valueType: builtinValueType: @int]',
               'typing.List[int]',
               [],
               "jsg.ArrayFactory('{name}', _CONTEXT, jsg.Integer, 0, None)",
               'None',
               [])
              )

        t = cast(JSGArrayExpr, parse("id [.] ", "arrayDef", JSGArrayExpr))
        self.check(t,
              ('arrayExpr: [valueType: builtinValueType: jsg.AnyType]',
               'typing.List[object]',
               [],
               "jsg.ArrayFactory('{name}', _CONTEXT, jsg.AnyTypeFactory('{name}', _CONTEXT), 0, None)",
               'None',
               [])
              )

        t = cast(JSGArrayExpr, parse("id [(Aa|BB|'foo')] ", "arrayDef", JSGArrayExpr))
        self.check(t,
              ("arrayExpr: [valueType: ((STRING: pattern: r'foo') | Undefined(Aa) | BB)]",
               'typing.List[typing.Union[str, Undefined(Aa), str]]',
               ['_Anon1', 'Aa', 'BB'],
               "jsg.ArrayFactory('{name}', _CONTEXT, typing.Union[_Anon1, Undefined(Aa), BB], 0, None)",
               'None',
               [])
              )

        t = cast(JSGArrayExpr, parse("id [(Aa|BB|'foo'){0,}] ", "arrayDef", JSGArrayExpr))
        self.check(t,
              ("arrayExpr: [valueType: ((STRING: pattern: r'foo') | Undefined(Aa) | BB){0,}]",
                'typing.List[typing.Union[str, Undefined(Aa), str]]',
               ['_Anon1', 'Aa', 'BB'],
               "jsg.ArrayFactory('{name}', _CONTEXT, typing.Union[_Anon1, Undefined(Aa), BB], 0, None)",
               'None',
               [])
              )

    def test_options(self):
        txt = "exclusions [(objectValue|LanguageStem) +]"
        t = cast(JSGArrayExpr, parse(txt, "arrayDef", JSGArrayExpr))
        self.check(t,
                   ('arrayExpr: [valueType: (Undefined(objectValue) | Undefined(LanguageStem))+]',
                    'typing.List[typing.Union[Undefined(objectValue), Undefined(LanguageStem)]]',
                    ['objectValue', 'LanguageStem'],
                    "jsg.ArrayFactory('{name}', _CONTEXT, typing.Union[Undefined(objectValue), Undefined(LanguageStem)], 1, None)",
                    'None',
                    [])
                   )

    def test_multi_types(self):
        txt = "x [objectValue | languageStem {3,7}]"
        t = cast(JSGArrayExpr, parse(txt, "arrayDef", JSGArrayExpr))
        self.check(t,
                   ('arrayExpr: [(valueType: ID: objectValue | valueType: ID: '
                    'languageStem){3,7}]',
                    'typing.List[typing.Union[Undefined(objectValue), Undefined(languageStem)]]',
                    ['objectValue', 'languageStem'],
                    "jsg.ArrayFactory('{name}', _CONTEXT, typing.Union[Undefined(objectValue), Undefined(languageStem)], 3, 7)",
                    'None',
                    [])
                   )


if __name__ == '__main__':
    unittest.main()




import unittest

from tests.test_python_generator.utils import PythonGeneratorUtils


class ListTestCase(PythonGeneratorUtils):
    save_output_files: bool = False

    def test_list(self):
        import tests.test_python_generator.py.list as doc

        jsg = 'l {e:@string*}'
        test_cases = [
            '{"e": []}',
            '{"e": ["abc"]}'
        ]
        self.do_test(jsg, 'list', doc, test_cases, {})

        import tests.test_python_generator.py.list_2 as doc2
        jsg2 = 'l {e:@string+}'
        test_cases2 = [
            '{"e": ["abc"]}'
        ]
        fail_test_cases2 = [
            '{}',
            '{"e": null}',
            '{"e": []}',
        ]
        self.do_test(jsg2, 'list_2', doc2, test_cases2, {}, fail_test_cases2)


if __name__ == '__main__':
    unittest.main()



import os
import unittest

from dict_compare import compare_dicts
from jsonasobj.jsonobj import as_json, as_dict

import tests.test_basics.py.ShExJ as ShExJ
from pyjsg.jsglib.loader import loads, is_valid, Logger
from jsonasobj import loads as jao_loads


class XSFacetTestCase(unittest.TestCase):
    def test_facet(self):
        file_loc = os.path.join(os.path.dirname(__file__), 'data', '1bnodeLength.json')
        with open(file_loc)  as f:
            text = f.read()
        facets = loads(text, ShExJ)
        # print(as_json(facets))
        log = Logger()
        val = is_valid(facets, log)
        # print(log.getvalue())
        self.assertTrue(val)
        d1 = jao_loads(text)
        d2 = jao_loads(as_json(facets))
        self.assertTrue(compare_dicts(as_dict(d1), as_dict(d2)))

if __name__ == '__main__':
    unittest.main()



import unittest

from pyjsg.validate_json import JSGPython


class MyTestCase(unittest.TestCase):
    def test_nested_objects(self):
        """ The issue being tested here is that we appear to be issuing anonymous identifiers for
        inner objects twice -- once when referenced and a second time when publishing.  As an example, the
        'entry' in 'directory' is typed as '_Anon1', while the ArrayFactory names it as '_Anon6'
        """
        jsg = """
        directory {
            entries: {
                id: ID,
                name: {
                    first: @string,
                    middle: [@string]?,
                    last: @string
                },
                address: {
                    city: @string,
                    state: @string,
                    zip: @int?
                }*
            }*
        }

        @terminals
        ID: [1-9][0-9]{7} @int;
        """

        p1 = '''
          {
            "entries": [
                {"id": 11725433,
                  "name": {
                      "first": "Sam",
                      "last": "Sneed"
                  },
                  "address": [
                    {
                      "city": "Southwark",
                      "state": "Bliss"
                    }
                  ]
                },
                {"id": 10000001,
                  "name": {
                      "first": "Julie",
                      "middle": ["Mary", "Elizabeth"],
                      "last": "Sneed"
                  },
                  "address": [
                    {
                      "city": "Southwark",
                      "state": "Agony"
                    }
                  ]
                }
            ]
          }  
        '''

        x = JSGPython(jsg, print_python=False)
        rslt = x.conforms(p1, 'p1')
        if not rslt.success:
            print(str(rslt))

        self.assertTrue(rslt.success)


if __name__ == '__main__':
    unittest.main()



import os
import unittest
from contextlib import redirect_stdout
from io import StringIO

from pyjsg.parser_impl.generate_python import generate


class EOptionTestCase(unittest.TestCase):
    def test_e_option(self):
        """ Test the '-e' option """
        shexj_jsg = os.path.join(os.path.dirname(__file__), '..', '..', 'tests', 'test_basics', 'jsg', 'ShExJ.jsg')
        outf = StringIO()
        with redirect_stdout(outf):
            # Should not fail
            self.assertEqual(0, generate([shexj_jsg, "-e"]))


if __name__ == '__main__':
    unittest.main()



import sys
import os
import unittest

from importlib import import_module

from pyjsg.jsglib.loader import load
from pyjsg.jsglib.logger import Logger
from pyjsg.parser_impl.generate_python import generate
from jsonasobj import load as json_load

log = Logger(sys.stdout)


class JSGReadMeTestCase(unittest.TestCase):

    log = Logger(sys.stdout)

    def eval_for_error(self, fn: str, mod, error: str):
        fn_in_json = json_load(fn)
        expected = fn_in_json._ERROR
        del fn_in_json._ERROR
        with self.assertRaises(ValueError) as context:
            r = load(fn, mod)
        self.assertEqual(expected, str(context.exception))

    def eval_python(self, cwd: str, dirpath: str, fn: str) -> None:
        basefile = fn.rsplit('.', 1)[0]
        outfile = os.path.abspath(os.path.join(cwd, "py", basefile + ".py"))
        self.assertEqual(0, generate([os.path.relpath(os.path.join(dirpath, fn)), "-o", outfile, "-nh"]))
        mod = import_module("tests_standalone.test_jsg_readme.py." + basefile)
        num_evaluated = 0
        for dirpath, _, filenames in os.walk(os.path.join(cwd, "json")):
            for filename in filenames:
                if filename.startswith(basefile) and filename.endswith(".json"):
                    num_evaluated += 1
                    full_fn = os.path.join(dirpath, filename)
                    if "_f" not in os.path.basename(full_fn):
                        r = load(full_fn, mod)
                        self.assertTrue(r._is_valid(log))
                    else:
                        self.eval_for_error(full_fn, mod, "ValueError: Unknown attribute: text=left in 2017")
        # TODO: complete this test
        # self.assertTrue(num_evaluated > 0, f"{fn} has no json equivalents")

    @unittest.skipIf(False, "This test has to be run alone -- namespace conflicts otherwise ")
    def test_jsg_readme(self):
        cwd = os.path.abspath(os.path.dirname(__file__))
        num_evaluated = 0
        for dirpath, _, filenames in os.walk(os.path.join(cwd, "jsg")):
            for fn in filenames:
                if fn.endswith(".jsg"):
                    num_evaluated += 1
                    self.eval_python(cwd, dirpath, fn)
        self.assertEqual(6, num_evaluated)

    @unittest.skipIf(True, "Outsdanding issue here on get")
    def test_gsg_1(self):
        # TODO: Fix this problem!
        self.assertTrue(False, "Rename ge1_f1.json to get.json")

if __name__ == '__main__':
    unittest.main()



import unittest
from typing import cast

from pyjsg.parser_impl.jsg_doc_parser import JSGDocParser
from pyjsg.parser_impl.jsg_pairdef_parser import JSGPairDef
from tests.test_basics.parser import parse

tests = [("x1 : INT", "INT", "str", ["INT"], ['x1: str = None'], [('x1', 'INT')],
          "pairDef: x1 : valueType: LEXER_ID_REF: INT"),
         ("x2 : INT?", "typing.Optional[INT]", "typing.Optional[str]", ["INT"], ['x2: typing.Optional[str] = None'], [
             ('x2', 'typing.Optional[INT]')], "pairDef: x2 : valueType: LEXER_ID_REF: INT?"),
         ("x3 : INT+", "jsg.ArrayFactory('{name}', _CONTEXT, INT, 1, None)", "typing.List[str]", ["INT"],
          ['x3: typing.List[str] = None'], [('x3', "jsg.ArrayFactory('x3', _CONTEXT, INT, 1, None)")],
          "pairDef: x3 : valueType: LEXER_ID_REF: INT+"),
         ("x4 : INT*", "jsg.ArrayFactory('{name}', _CONTEXT, INT, 0, None)", "typing.List[str]",
          ["INT"], ['x4: typing.List[str] = None'], [("x4", "jsg.ArrayFactory('x4', _CONTEXT, INT, 0, None)")],
          "pairDef: x4 : valueType: LEXER_ID_REF: INT*"),
         ("x5 : INT{1}", "INT",  "str", ["INT"], ['x5: str = None'], [("x5", "INT")],
          "pairDef: x5 : valueType: LEXER_ID_REF: INT{1}"),
         ("x6 : INT{1, 1}", "INT", "str", ["INT"], ['x6: str = None'], [("x6", "INT")],
          "pairDef: x6 : valueType: LEXER_ID_REF: INT{1,1}"),
         ("x7 : INT{1,}", "jsg.ArrayFactory('{name}', _CONTEXT, INT, 1, None)",
          "typing.List[str]", ["INT"], ['x7: typing.List[str] = None'], [("x7", "jsg.ArrayFactory('x7', _CONTEXT, INT, 1, None)")],
          "pairDef: x7 : valueType: LEXER_ID_REF: INT{1,}")]

initializers = {
    "x1 : INT": ['self.x1 = x1'],
    "x2 : INT?": ['self.x2 = x2'],
    "x3 : INT+": ["self.x3 = x3"],
    "x4 : INT*": ["self.x4 = x4"],
    "x5 : INT{1}": ['self.x5 = x5'],
    "x6 : INT{1, 1}": ['self.x6 = x6'],
    "x7 : INT{1,}": ["self.x7 = x7"]
}

builtins = [("k : @string", ["k: str = None"], ['self.k = k'],
             'pairDef: k : valueType: builtinValueType: @string')]


class PairDefTestCase(unittest.TestCase):
    def test_simple_pairdef(self):
        for text, sig, py, deps, sigs, memins, v in tests:
            t = cast(JSGPairDef, parse(text, "pairDef", JSGPairDef))
            self.assertEqual(v, str(t))
            self.assertEqual(sig, t.signature_type(), text)
            self.assertEqual(py, t.python_type(), text)
            self.assertEqual(deps, t.dependency_list(), text)
            self.assertEqual(sigs, t.signatures(), text)
            self.assertEqual(memins, t.members_entries(), text)
            self.assertEqual(initializers[text], t.initializers())

    def test_odd_identifiers(self):
        t = cast(JSGPairDef, parse("'String' : INT", "pairDef", JSGPairDef))
        self.assertEqual('pairDef: String : valueType: LEXER_ID_REF: INT', str(t))
        self.assertEqual([('String', 'INT')], t.members_entries())
        self.assertEqual([('String', 'typing.Optional[INT]')], t.members_entries(True))
        self.assertEqual(['String: str = None'], t.signatures())
        self.assertEqual(['self.String = String'], t.initializers())
        self.assertEqual("pairDef: String : valueType: LEXER_ID_REF: INT", str(t))

        t = cast(JSGPairDef, parse("'def' : INT", "pairDef", JSGPairDef))
        self.assertEqual('pairDef: def : valueType: LEXER_ID_REF: INT', str(t))
        self.assertEqual([('def', 'INT')], t.members_entries())
        self.assertEqual(['def_: str = None'], t.signatures())
        self.assertEqual(["setattr(self, 'def', def_ if def_ is not None else _kwargs.get('def', None))"],
                         t.initializers())
        self.assertEqual("pairDef: def : valueType: LEXER_ID_REF: INT", str(t))

        t = cast(JSGPairDef, parse("'a var' : @number", "pairDef", JSGPairDef))
        self.assertEqual("pairDef: a var : valueType: builtinValueType: @number", str(t))
        self.assertEqual([('a var', 'jsg.Number')], t.members_entries())
        self.assertEqual([], t.signatures())
        self.assertEqual(["setattr(self, 'a var', _kwargs.get('a var', None))"], t.initializers())

    def test_pairdef_shorthand(self):
        text = "(x1 'v 2' 'class') : @number {3,17}"
        t = cast(JSGPairDef, parse(text, "pairDef", JSGPairDef))
        self.assertEqual(['x1: typing.List[float] = None', 'class_: typing.List[float] = None'], t.signatures())
        self.assertEqual([
            'self.x1 = x1',
            "setattr(self, 'v 2', _kwargs.get('v 2', None))",
            "setattr(self, 'class', class_ if class_ is not None else "
            "_kwargs.get('class', None))"],
                         t.initializers())
        self.assertEqual("pairDef: (x1 | v 2 | class) : valueType: builtinValueType: @number{3,17}", str(t))
        self.assertEqual([], t.dependency_list())

        text = "(x1 'v 2' 'class') : @bool ?"
        t = cast(JSGPairDef, parse(text, "pairDef", JSGPairDef))
        self.assertEqual(['x1: typing.Optional[bool] = None', 'class_: typing.Optional[bool] = None'], t.signatures())
        self.assertEqual([
            'self.x1 = x1',
            "setattr(self, 'v 2', _kwargs.get('v 2', None))",
            "setattr(self, 'class', class_ if class_ is not None else _kwargs.get('class', None))"], t.initializers())
        self.assertEqual("pairDef: (x1 | v 2 | class) : valueType: builtinValueType: @bool?", str(t))
        self.assertEqual([], t.dependency_list())
        self.assertEqual([('x1', 'typing.Optional[jsg.Boolean]'),
                          ('v 2', 'typing.Optional[jsg.Boolean]'),
                          ('class', 'typing.Optional[jsg.Boolean]')], t.members_entries())

        text = "(x1 'v 2' 'class') : @null"
        t = cast(JSGPairDef, parse(text, "pairDef", JSGPairDef))
        self.assertEqual(['x1: type(None) = jsg.Empty', 'class_: type(None) = jsg.Empty'], t.signatures())
        self.assertEqual([
            'self.x1 = x1',
             "setattr(self, 'v 2', _kwargs.get('v 2', jsg.Empty))",
             "setattr(self, 'class', class_ if class_ is not jsg.Empty else _kwargs.get('class', jsg.Empty))"],
            t.initializers())
        self.assertEqual("pairDef: (x1 | v 2 | class) : valueType: builtinValueType: @null", str(t))
        self.assertEqual([], t.dependency_list())
        self.assertEqual([('x1', 'jsg.JSGNull'), ('v 2', 'jsg.JSGNull'), ('class', 'jsg.JSGNull')], t.members_entries())
        text = "(def 'v 2' class) : @null"
        t = cast(JSGPairDef, parse(text, "pairDef", JSGPairDef))
        self.assertEqual([
            "setattr(self, 'def', def_ if def_ is not jsg.Empty else _kwargs.get('def', "
             'jsg.Empty))',
             "setattr(self, 'v 2', _kwargs.get('v 2', jsg.Empty))",
             "setattr(self, 'class', class_ if class_ is not jsg.Empty else "
             "_kwargs.get('class', jsg.Empty))"], t.initializers())

    def test_pairdef_valuetype_ref(self):
        text = "nonobj = {a:@string b:@number?};  obj = {nonobj}"
        t = cast(JSGPairDef, parse(text, "grammarElt", JSGPairDef))
        self.assertEqual(['a: typing.Optional[float] = None', 'b: typing.Optional[float] = None'], t.signatures())

    def test_pairdef_builtins(self):
        for text, sig, init, s in builtins:
            t = cast(JSGPairDef, parse(text, "pairDef", JSGPairDef))
            self.assertEqual(sig, t.signatures())
            self.assertEqual(init, t.initializers())
            self.assertEqual(s, str(t))
            self.assertEqual([], t.dependency_list())
            self.assertEqual([('k', 'jsg.String')], t.members_entries())

    def test_pairdef_reference(self):
        text = """
Patient {name: @string+ age: @int}
b {Patient*}
"""
        d = cast(JSGDocParser, parse(text, "doc", JSGDocParser))
        self.assertIsNotNone(d)
        t = d._context.reference('b')
        self.assertEqual('objectExpr: simple object', str(t))
        self.assertEqual(['name: typing.List[str] = None', 'age: int = None'], t.signatures())
        self.assertEqual(['self.name = name', 'self.age = age'], t.initializers())
        self.assertEqual(['Patient'], t.dependency_list())
        self.assertEqual([
            ('name', "jsg.ArrayFactory('name', _CONTEXT, jsg.ArrayFactory('name', _CONTEXT, jsg.String, 1, None), 0, None)"),
             ('age', "jsg.ArrayFactory('age', _CONTEXT, jsg.Integer, 0, None)")], t.members_entries())
        self.assertEqual('None', t.mt_value())


if __name__ == '__main__':
    unittest.main()



import unittest

from pyjsg.validate_json import JSGPython


class ExtraTestCase(unittest.TestCase):
    def test_extra(self):
        x = JSGPython('doc {a:@string}')
        self.assertFalse(x.conforms('{"a":"hello", "target":"earthling"}').success)


if __name__ == '__main__':
    unittest.main()



import unittest
from typing import cast

from pyjsg.parser_impl.jsg_objectexpr_parser import JSGObjectExpr
from pyjsg.parser_impl.jsg_valuetype_parser import JSGValueType
from tests.test_basics.parser import parse

builtin_tests = [("@string", "jsg.String", "str", "None"),
                 ("@number", "jsg.Number", "float", "None"),
                 ("@int", "jsg.Integer", "int", "None"),
                 ("@bool", "jsg.Boolean", "bool", "None"),
                 ("@null", "jsg.JSGNull", "type(None)", "jsg.Empty"),
                 ("@array", "jsg.ArrayFactory('{name}', _CONTEXT, jsg.AnyType, 0, None)", "list", "None"),
                 ("@object", "jsg.ObjectFactory('{name}', _CONTEXT, jsg.Object)", "object", "None"),
                 (".", "jsg.AnyTypeFactory('{name}', _CONTEXT)", "object", "jsg.Empty")]

ref_tests = [("A", "ID"),
             ("a", "ID"),
             ("ABCt", "ID"),
             ("aNID", "ID"),
             ("AA", "LEXER_ID_REF"),
             ("HEX_NUM", "LEXER_ID_REF")]

constructor_tests = [
    ("@string", "jsg.String(getter('v 1', None))"),
    ("@number", "jsg.Number(getter('v 1', None))"),
    ("@int", "jsg.Integer(getter('v 1', None))"),
    ("@bool", "jsg.Boolean(getter('v 1', None))"),
    ("@null", "jsg.JSGNull(getter('v 1', None))"),
    ("@array", "Array('k', self._context, jsg.AnyType, 0, None, getter('v 1', None))"),
    ("@object", "jsg.Object('k', self._context, getter('v 1', None))"),
    (".", "jsg.AnyType('k', self._context, getter('v 1', None))")]


class ValueTypeTestCase(unittest.TestCase):
    def test_builtins(self):
        for text, sig, typ_, mt_typ in builtin_tests:
            t = cast(JSGValueType, parse(text, "valueType", JSGValueType))
            self.assertEqual(sig, t.signature_type(), text)
            self.assertEqual(typ_, t.python_type(), text)
            self.assertEqual(f"valueType: builtinValueType: {'jsg.AnyType' if text == '.' else text}", str(t), text)
            self.assertEqual(mt_typ, t.mt_value(), text)
            self.assertEqual([], t.members_entries(), text)
            self.assertEqual([], t.dependency_list(), text)

    def test_refs(self):
        for text, typ in ref_tests:
            t = cast(JSGValueType, parse(text, "valueType", JSGValueType))
            self.assertEqual(f'Undefined({text})' if typ != 'LEXER_ID_REF' else text, t.signature_type())
            self.assertEqual(f'Undefined({text})' if typ != 'LEXER_ID_REF' else 'str', t.python_type())
            self.assertEqual(f"valueType: {typ}: {text}", str(t))
            self.assertEqual([text], t.dependency_list())
            self.assertEqual([], t.members_entries())
            self.assertEqual("None", t.mt_value())

    def test_ref_escapes(self):
        self.assertEqual("Undefined(Class)", cast(JSGValueType, parse("Class", "valueType", JSGValueType))
                         .signature_type())
        t = cast(JSGValueType, parse("class", "valueType", JSGValueType))
        self.assertEqual("Undefined(class_)", t.signature_type())
        self.assertEqual("Undefined(class_)", t.python_type())
        self.assertEqual([], t.members_entries())

    def test_literals(self):
        t = cast(JSGValueType, parse("'literal'", "valueType", JSGValueType))
        self.assertEqual("_Anon1", t.signature_type())
        self.assertEqual("str", t.python_type())
        self.assertEqual("valueType: STRING: pattern: r'literal'", str(t))

    def test_any(self):
        t = cast(JSGValueType, parse("id = .;", "valueTypeMacro", JSGValueType))
        self.assertEqual("object", t.python_type())
        self.assertEqual("jsg.AnyTypeFactory('{name}', _CONTEXT)", t.signature_type())
        self.assertEqual("jsg.Empty", t.mt_value())
        self.assertEqual([], t.members_entries())
        self.assertEqual([], t.dependency_list())
        self.assertEqual("valueType: builtinValueType: jsg.AnyType", str(t))

    def test_alternatives(self):
        t = cast(JSGValueType, parse("id = ('x'|'y') ;", "valueTypeMacro", JSGValueType))
        self.assertEqual("_Anon1", t.signature_type())
        self.assertEqual("str", t.python_type())
        self.assertEqual("valueType: STRING: pattern: r'(x)|(y)'", str(t))
        t = cast(JSGValueType, parse("id = (Aa | Bb | (Cc | Dd)) ;", "valueTypeMacro", JSGValueType))
        self.assertEqual("typing.Union[Undefined(Aa), Undefined(Bb), typing.Union[Undefined(Cc), Undefined(Dd)]]",
                         t.signature_type())
        self.assertEqual("typing.Union[Undefined(Aa), Undefined(Bb), typing.Union[Undefined(Cc), Undefined(Dd)]]",
                         t.python_type())
        self.assertEqual(t.dependency_list(), ['Aa', 'Bb', 'Cc', 'Dd'])
        self.assertEqual("valueType: (Undefined(Aa) | Undefined(Bb) | typing.Union[Undefined(Cc), Undefined(Dd)])",
                         str(t))
        self.assertEqual([], t.members_entries())
        t = cast(JSGValueType, parse("id = (Aa | Bb | 'foo' | (Cc | Dd) | 'bar') ;", "valueTypeMacro", JSGValueType))
        self.assertEqual("typing.Union[_Anon1, Undefined(Aa), Undefined(Bb), "
                         "typing.Union[Undefined(Cc), Undefined(Dd)]]",
                         t.signature_type())
        self.assertEqual("typing.Union[str, Undefined(Aa), Undefined(Bb), typing.Union[Undefined(Cc), Undefined(Dd)]]",
                         t.python_type())
        self.assertEqual(['_Anon1', 'Aa', 'Bb', 'Cc', 'Dd'], t.dependency_list())
        self.assertEqual("valueType: ((STRING: pattern: r'(foo)|(bar)') | Undefined(Aa) | Undefined(Bb) | "
                         "typing.Union[Undefined(Cc), Undefined(Dd)])", str(t))

    def test_array(self):
        t = cast(JSGValueType, parse('id = [.] ;', "valueTypeMacro", JSGValueType))
        self.assertEqual('valueType: arrayExpr: [valueType: builtinValueType: jsg.AnyType]', str(t))
        self.assertEqual([], t.dependency_list())
        self.assertEqual([], t.members_entries())
        self.assertEqual('typing.List[object]', t.python_type())
        self.assertEqual("jsg.ArrayFactory('{name}', _CONTEXT, jsg.AnyTypeFactory('{name}', _CONTEXT), 0, None)",
                         t.signature_type())
        self.assertEqual('None', t.mt_value())

        t = cast(JSGValueType, parse('id = [@int | "AB*" +] ;', "valueTypeMacro", JSGValueType))
        self.assertEqual("valueType: arrayExpr: [(valueType: builtinValueType: "
                         "@int | valueType: STRING: pattern: r'AB\*')+]", str(t))
        self.assertEqual(['_Anon1'], t.dependency_list())
        self.assertEqual([], t.members_entries())
        self.assertEqual('typing.List[typing.Union[int, str]]', t.python_type())
        self.assertEqual("jsg.ArrayFactory('{name}', _CONTEXT, typing.Union[jsg.Integer, _Anon1], 1, None)",
                         t.signature_type())
        self.assertEqual('None', t.mt_value())

    def test_lexeridref(self):
        t = cast(JSGValueType, parse('("[a-z]*" | "0-9*")', "valueType", JSGValueType))
        self.assertEqual("valueType: STRING: pattern: r'(\[a\-z\]\*)|(0\-9\*)'", str(t))
        self.assertEqual("_Anon1", t.signature_type())
        self.assertEqual("str", t.python_type())
        self.assertEqual("None", t.mt_value())
        self.assertEqual([], t.members_entries())

        t = cast(JSGValueType, parse('("[a-z]*" | "0-9*" | ID)', "valueType", JSGValueType))
        self.assertEqual("valueType: ((STRING: pattern: r'(\[a\-z\]\*)|(0\-9\*)') | ID)", str(t))
        self.assertEqual("typing.Union[_Anon1, ID]", t.signature_type())
        self.assertEqual("typing.Union[str, str]", t.python_type())
        self.assertEqual("None", t.mt_value())
        self.assertEqual([], t.members_entries())

    def test_anon_typeid(self):
        t = cast(JSGValueType, parse("{a: @int b: @string+}", "valueType", JSGValueType))
        self.assertEqual('valueType: (anonymous: _Anon1): objectExpr: simple object', str(t))
        self.assertEqual('_Anon1', t.signature_type())
        self.assertEqual('_Anon1', t.python_type())
        self.assertEqual('None', t.mt_value())
        self.assertEqual([], t.members_entries())

    def test_objectmacro_opts(self):
        t = cast(JSGValueType, parse("a = @string | KT | {} ;", "valueTypeMacro", JSGValueType))
        self.assertEqual('valueType: (jsg.String | KT | _Anon1)', str(t))
        self.assertEqual('typing.Union[jsg.String, KT, _Anon1]', t.signature_type())
        self.assertEqual('typing.Union[str, str, _Anon1]', t.python_type())
        self.assertEqual('None', t.mt_value())
        self.assertEqual([], t.members_entries())

    def test_objectmacro(self):
        t = cast(JSGObjectExpr, parse("stringFacet = (length minlength maxlength):"
                                      "INTEGER pattern:STRING flags:STRING? ;", "objectMacro", JSGObjectExpr))
        self.assertEqual("""class stringFacet(jsg.JSGObject):
    _reference_types = []
    _members = {'length': INTEGER,
                'minlength': INTEGER,
                'maxlength': INTEGER,
                'pattern': STRING,
                'flags': typing.Optional[STRING]}
    _strict = True

    def __init__(self,
                 length: str = None,
                 minlength: str = None,
                 maxlength: str = None,
                 pattern: str = None,
                 flags: typing.Optional[str] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.length = length
        self.minlength = minlength
        self.maxlength = maxlength
        self.pattern = pattern
        self.flags = flags""", t.as_python('stringFacet').strip())
        self.assertEqual(['INTEGER', 'STRING'], t.dependency_list())

        t = cast(JSGObjectExpr, parse("stringFacet = (length minlength maxlength):INTEGER "
                                      "pattern:STRING | flags:STRING? ;", "objectMacro", JSGObjectExpr))
        self.assertEqual("""class stringFacet(jsg.JSGObject):
    _reference_types = [_Anon1_1_, _Anon1_2_]
    _members = {'length': typing.Optional[INTEGER],
                'minlength': typing.Optional[INTEGER],
                'maxlength': typing.Optional[INTEGER],
                'pattern': typing.Optional[STRING],
                'flags': typing.Optional[STRING]}
    _strict = True

    def __init__(self,
                 opts_: typing.Union[_Anon1_1_, _Anon1_2_] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        if opts_ is not None:
            if isinstance(opts_, _Anon1_1_):
                self.length = opts_.length
                self.minlength = opts_.minlength
                self.maxlength = opts_.maxlength
                self.pattern = opts_.pattern
            elif isinstance(opts_, _Anon1_2_):
                self.flags = opts_.flags
            else:
                raise ValueError(f"Unrecognized value type: {opts_}")""", t.as_python('stringFacet').strip())
        self.assertEqual(['INTEGER', 'STRING', '_Anon1_1_', '_Anon1_2_'], t.dependency_list())

        t = cast(JSGObjectExpr, parse("x = a:@number | b:@null | ;", "objectMacro", JSGObjectExpr))
        self.assertEqual("""class stringFacet(jsg.JSGObject):
    _reference_types = [_Anon1_1_, _Anon1_2_, _Anon1_3_]
    _members = {'a': typing.Optional[jsg.Number],
                'b': typing.Optional[jsg.JSGNull]}
    _strict = True

    def __init__(self,
                 opts_: typing.Union[_Anon1_1_, _Anon1_2_, _Anon1_3_] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        if opts_ is not None:
            if isinstance(opts_, _Anon1_1_):
                self.a = opts_.a
            elif isinstance(opts_, _Anon1_2_):
                self.b = opts_.b
            elif isinstance(opts_, _Anon1_3_):
                pass
            else:
                raise ValueError(f"Unrecognized value type: {opts_}")""",
                         t.as_python('stringFacet').strip())
        self.assertEqual(['_Anon1_1_', '_Anon1_2_', '_Anon1_3_'], t.dependency_list())


if __name__ == '__main__':
    unittest.main()



import os
import unittest
from contextlib import redirect_stdout
from io import StringIO

files = []


class JSGTestCase(unittest.TestCase):
    def test_input_files(self):
        from pyjsg.parser_impl.generate_python import generate
        base = os.path.abspath(os.path.dirname(__file__))
        ntested = 0
        for dirpath, _, filenames in os.walk(os.path.join(base, "jsg")):
            outf = StringIO()
            with redirect_stdout(outf):
                for fn in filenames:
                    if fn.endswith(".jsg") and (not files or fn in files):
                        infile = os.path.relpath(os.path.join(dirpath, fn))
                        outfile = os.path.relpath(os.path.join(base, "py", fn.rsplit('.', 1)[0] + '.py'))
                        self.assertEqual(0, generate([infile, "-o", outfile, "-e", "-nh"]))
                        ntested += 1
        if files:
            print("WARNING: all jsg files were not tested")
            self.assertEqual(len(files), ntested)
        else:
            self.assertEqual(24, ntested)

if __name__ == '__main__':
    unittest.main()



import unittest
from typing import Union

from pyjsg.jsglib import String, JSGPattern, Number, Integer, Boolean, JSGNull, Empty


class JSGBuiltinsTestCase(unittest.TestCase):
    def do_test(self,
                cls,
                val: Union[str, int, float, bool, None],
                rslt: Union[str, int, float, bool]=None,
                fail: bool=False) -> Union[str, int, float, bool]:
        if fail:
            with self.assertRaises(ValueError):
                cls(val)
        else:
            t = cls(val)
            self.assertEqual(rslt if rslt is not None else val, t)
            if not isinstance(t, bool):
                self.assertEqual(rslt if rslt is not None else val, t.val)
            return t

    def test_string(self):
        self.do_test(String, "a simple string")
        self.do_test(String, True, "true", fail=True)
        self.do_test(String, False, "false", fail=True)
        self.do_test(String, 143, "143", fail=True)
        self.do_test(String, 95.221E+5, "9522100.0", fail=True)
        self.do_test(String, "95.221E+5")
        self.do_test(String, "^<80>߿ࠀ࿿က쿿퀀퟿�𐀀𿿽񀀀󿿽􀀀􏿽$/")
        self.do_test(String, "^\/\t\n\r\-\\\u0061\U0001D4B8$", "^\\/\t\n\r\\-\\a𝒸$")
        self.do_test(String, None, fail=True)
        self.do_test(String, -119, "-119", fail=True)

        class PAT_STR(String):
            pattern = JSGPattern(r'[a-z][0-9]+')

        self.do_test(PAT_STR, 'a143')

        with self.assertRaises(ValueError):
            x = PAT_STR('17')

        class INT_STR(String):
            pattern = JSGPattern(r'0|([1-9][0-9]*)')

        self.do_test(INT_STR, "17")
        # self.do_test(INT_STR, 17, "17")
        with self.assertRaises(ValueError):
            x = INT_STR("-17")
        with self.assertRaises(ValueError):
            x = INT_STR("something")

    def test_number(self):
        self.assertIsInstance(self.do_test(Number, 42), float)
        self.assertIsInstance(self.do_test(Number, -173), float)
        self.assertIsInstance(self.do_test(Number, 0.1723), float)
        with self.assertRaises(ValueError):
            x = Number("+173.0003E-5")          # JSON doesn't allow plus signs
        self.do_test(Number, "-173.0003E-5", -0.001730003)

        class POS_NUMBER(Number):
            pattern = JSGPattern(r'(0|[1-9][0-9]*)(.[0-9]+)?([eE][+-]?[0-9]+)?')

        self.do_test(POS_NUMBER, 1003675)
        with self.assertRaises(ValueError):
            x = POS_NUMBER(-1)
        with self.assertRaises(ValueError):
            x = POS_NUMBER("-117.438")

    def test_integer(self):
        self.assertIsInstance(self.do_test(Integer, 42), int)
        self.assertIsInstance(self.do_test(Integer, -173), int)
        self.assertIsInstance(self.do_test(Integer, 0), int)
        self.assertIsInstance(self.do_test(Integer, "-119221", -119221), int)
        with self.assertRaises(ValueError):
            x = Integer(0.1723)
        with self.assertRaises(ValueError):
            x = Integer("-173.0003E-5")
        with self.assertRaises(ValueError):
            x = Integer("a")
        with self.assertRaises(ValueError):
            x = Integer("")
        with self.assertRaises(ValueError):
            x = Integer(False)
        with self.assertRaises(ValueError):
            x = Integer(None)

        class NEG_INTEGER(Integer):
            pattern = JSGPattern(r'-(0|[1-9][0-9]*)')

        self.assertIsInstance(self.do_test(NEG_INTEGER, -119), int)
        with self.assertRaises(ValueError):
            x = NEG_INTEGER(17)

    def test_bool(self):
        self.assertIsInstance(self.do_test(Boolean, True), bool)
        self.assertIsInstance(self.do_test(Boolean, "True", True), bool)
        self.assertIsInstance(self.do_test(Boolean, "true", True), bool)
        self.assertIsInstance(self.do_test(Boolean, False), bool)
        self.assertIsInstance(self.do_test(Boolean, "False", False), bool)
        self.assertIsInstance(self.do_test(Boolean, "false", False), bool)
        with self.assertRaises(ValueError):
            Boolean(None)
        with self.assertRaises(ValueError):
            x = Boolean(0)
        with self.assertRaises(ValueError):
            x = Boolean("")

        class TRUE_ONLY(Boolean):
            pattern = Boolean.true_pattern

        self.assertIsInstance(self.do_test(TRUE_ONLY, True), bool)
        with self.assertRaises(ValueError):
            x = TRUE_ONLY(False)

    def test_incompatible_pattern(self):
        class INCOMPAT(Integer):
            pattern = JSGPattern(r'[a-z]+')

        with self.assertRaises(ValueError):
            x = INCOMPAT("a")

        with self.assertRaises(ValueError):
            x = INCOMPAT(17)

    def test_null(self):
        self.assertIsNone(JSGNull(None).val)
        self.assertIsNone(JSGNull(JSGNull).val)
        with self.assertRaises(ValueError):
            JSGNull(Empty)
        with self.assertRaises(ValueError):
            JSGNull('null')

    def test_empty_type(self):
        """ Make sure that Empty is a class only thingie """
        self.assertTrue(Empty is Empty())

if __name__ == '__main__':
    unittest.main()



import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()



class PN_CHARS_BASE(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[A-Z]|[a-z]|[\u00C0-\u00D6]|[\u00D8-\u00F6]|[\u00F8-\u02FF]|[\u0370-\u037D]|[\u037F-\u1FFF]|[\u200C-\u200D]|[\u2070-\u218F]|[\u2C00-\u2FEF]|[\u3001-\uD7FF]|[\uF900-\uFDCF]|[\uFDF0-\uFFFD]|[\u10000-\uEFFFF]')


class HEX(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[0-9]|[A-F]|[a-f]')


class PN_CHARS_U(jsg.JSGString):
    pattern = jsg.JSGPattern(r'({PN_CHARS_BASE})|_'.format(PN_CHARS_BASE=PN_CHARS_BASE.pattern))


class UCHAR(jsg.JSGString):
    pattern = jsg.JSGPattern(r'\\\\u({HEX})({HEX})({HEX})({HEX})|\\\\U({HEX})({HEX})({HEX})({HEX})({HEX})({HEX})({HEX})({HEX})'.format(HEX=HEX.pattern))


class PN_CHARS(jsg.JSGString):
    pattern = jsg.JSGPattern(r'({PN_CHARS_U})|\-|[0-9]|\\u00B7|[\u0300-\u036F]|[\u203F-\u2040]'.format(PN_CHARS_U=PN_CHARS_U.pattern))


class PN_PREFIX(jsg.JSGString):
    pattern = jsg.JSGPattern(r'({PN_CHARS_BASE})((({PN_CHARS})|\.)*({PN_CHARS}))?'.format(PN_CHARS=PN_CHARS.pattern, PN_CHARS_BASE=PN_CHARS_BASE.pattern))

_CONTEXT.NAMESPACE = locals()




import unittest

from pyjsg.parser_impl.anonymousidentifierfactory import AnonymousIdentifierFactory


class AnonIdTestCase(unittest.TestCase):
    from pyjsg.parser_impl.anonymousidentifierfactory import AnonymousIdentifierFactory

    def test1(self):
        fact = AnonymousIdentifierFactory()
        self.assertEqual("_Anon1", fact.next_id())
        self.assertEqual("_Anon2", fact.next_id())
        self.assertTrue(fact.is_anon("_Anon173"))
        self.assertFalse(fact.is_anon("_Anon01"))
        self.assertFalse(fact.is_anon("_Anona1"))
        fact = AnonymousIdentifierFactory("ID")
        self.assertEqual("ID1", fact.next_id())
        self.assertEqual("ID2", fact.next_id())
        self.assertTrue(fact.is_anon("ID1173"))
        self.assertFalse(fact.is_anon("ID01"))
        self.assertFalse(fact.is_anon("_Anon1"))


if __name__ == '__main__':
    unittest.main()



import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("stringFacet_1_")
_CONTEXT.TYPE_EXCEPTIONS.append("stringFacet_2_")
_CONTEXT.TYPE_EXCEPTIONS.append("numericFacet")
_CONTEXT.TYPE_EXCEPTIONS.append("xsFacet_2_")
_CONTEXT.TYPE_EXCEPTIONS.append("stringFacet")
_CONTEXT.TYPE_EXCEPTIONS.append("xsFacet_1_")
_CONTEXT.TYPE_EXCEPTIONS.append("xsFacet")
_CONTEXT.TYPE_EXCEPTIONS.append("NodeConstraint")


class _Anon1(jsg.JSGString):
    pattern = jsg.JSGPattern(r'(iri)|(bnode)|(nonliteral)|(literal)')


class IRI(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[0-9]')


class stringFacet_1_(jsg.JSGObject):
    _reference_types = []
    _members = {'length': jsg.Integer,
                'minlength': jsg.Integer,
                'maxlength': jsg.Integer}
    _strict = True

    def __init__(self,
                 length: int = None,
                 minlength: int = None,
                 maxlength: int = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.length = length
        self.minlength = minlength
        self.maxlength = maxlength



class stringFacet_2_(jsg.JSGObject):
    _reference_types = []
    _members = {'pattern': jsg.String,
                'flags': typing.Optional[jsg.String]}
    _strict = True

    def __init__(self,
                 pattern: str = None,
                 flags: typing.Optional[str] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.pattern = pattern
        self.flags = flags



class numericFacet(jsg.JSGObject):
    _reference_types = []
    _members = {'mininclusive': jsg.Integer,
                'minexclusive': jsg.Integer,
                'maxinclusive': jsg.Integer,
                'maxexclusive': jsg.Integer,
                'totaldigits': jsg.Integer,
                'fractiondigits': jsg.Integer}
    _strict = True

    def __init__(self,
                 mininclusive: int = None,
                 minexclusive: int = None,
                 maxinclusive: int = None,
                 maxexclusive: int = None,
                 totaldigits: int = None,
                 fractiondigits: int = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.mininclusive = mininclusive
        self.minexclusive = minexclusive
        self.maxinclusive = maxinclusive
        self.maxexclusive = maxexclusive
        self.totaldigits = totaldigits
        self.fractiondigits = fractiondigits



class xsFacet_2_(jsg.JSGObject):
    _reference_types = [numericFacet]
    _members = {'mininclusive': jsg.Integer,
                'minexclusive': jsg.Integer,
                'maxinclusive': jsg.Integer,
                'maxexclusive': jsg.Integer,
                'totaldigits': jsg.Integer,
                'fractiondigits': jsg.Integer}
    _strict = True

    def __init__(self,
                 mininclusive: int = None,
                 minexclusive: int = None,
                 maxinclusive: int = None,
                 maxexclusive: int = None,
                 totaldigits: int = None,
                 fractiondigits: int = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.mininclusive = mininclusive
        self.minexclusive = minexclusive
        self.maxinclusive = maxinclusive
        self.maxexclusive = maxexclusive
        self.totaldigits = totaldigits
        self.fractiondigits = fractiondigits



class stringFacet(jsg.JSGObject):
    _reference_types = [stringFacet_1_, stringFacet_2_]
    _members = {'length': typing.Optional[jsg.Integer],
                'minlength': typing.Optional[jsg.Integer],
                'maxlength': typing.Optional[jsg.Integer],
                'pattern': typing.Optional[jsg.String],
                'flags': typing.Optional[jsg.String]}
    _strict = True

    def __init__(self,
                 opts_: typing.Union[stringFacet_1_, stringFacet_2_] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        if opts_ is not None:
            if isinstance(opts_, stringFacet_1_):
                self.length = opts_.length
                self.minlength = opts_.minlength
                self.maxlength = opts_.maxlength
            elif isinstance(opts_, stringFacet_2_):
                self.pattern = opts_.pattern
                self.flags = opts_.flags
            else:
                raise ValueError(f"Unrecognized value type: {opts_}")



class xsFacet_1_(jsg.JSGObject):
    _reference_types = [stringFacet]
    _members = {'length': typing.Optional[jsg.Integer],
                'minlength': typing.Optional[jsg.Integer],
                'maxlength': typing.Optional[jsg.Integer],
                'pattern': typing.Optional[jsg.String],
                'flags': typing.Optional[jsg.String]}
    _strict = True

    def __init__(self,
                 opts_: typing.Union[stringFacet_1_, stringFacet_2_] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        if opts_ is not None:
            if isinstance(opts_, stringFacet_1_):
                self.length = opts_.length
                self.minlength = opts_.minlength
                self.maxlength = opts_.maxlength
            elif isinstance(opts_, stringFacet_2_):
                self.pattern = opts_.pattern
                self.flags = opts_.flags
            else:
                raise ValueError(f"Unrecognized value type: {opts_}")



class xsFacet(jsg.JSGObject):
    _reference_types = [xsFacet_1_, xsFacet_2_]
    _members = {'length': typing.Optional[typing.Optional[jsg.Integer]],
                'minlength': typing.Optional[typing.Optional[jsg.Integer]],
                'maxlength': typing.Optional[typing.Optional[jsg.Integer]],
                'pattern': typing.Optional[typing.Optional[jsg.String]],
                'flags': typing.Optional[typing.Optional[jsg.String]],
                'mininclusive': typing.Optional[typing.Optional[jsg.Integer]],
                'minexclusive': typing.Optional[typing.Optional[jsg.Integer]],
                'maxinclusive': typing.Optional[typing.Optional[jsg.Integer]],
                'maxexclusive': typing.Optional[typing.Optional[jsg.Integer]],
                'totaldigits': typing.Optional[typing.Optional[jsg.Integer]],
                'fractiondigits': typing.Optional[typing.Optional[jsg.Integer]]}
    _strict = True

    def __init__(self,
                 opts_: typing.Union[xsFacet_1_, xsFacet_2_] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        if opts_ is not None:
            if isinstance(opts_, xsFacet_1_):
                if opts_ is not None:
                    if isinstance(opts_, stringFacet_1_):
                        self.length = opts_.length
                        self.minlength = opts_.minlength
                        self.maxlength = opts_.maxlength
                    elif isinstance(opts_, stringFacet_2_):
                        self.pattern = opts_.pattern
                        self.flags = opts_.flags
                    else:
                        raise ValueError(f"Unrecognized value type: {opts_}")
            elif isinstance(opts_, xsFacet_2_):
                self.mininclusive = opts_.mininclusive
                self.minexclusive = opts_.minexclusive
                self.maxinclusive = opts_.maxinclusive
                self.maxexclusive = opts_.maxexclusive
                self.totaldigits = opts_.totaldigits
                self.fractiondigits = opts_.fractiondigits
            else:
                raise ValueError(f"Unrecognized value type: {opts_}")



class NodeConstraint(jsg.JSGObject):
    _reference_types = [xsFacet]
    _members = {'nodeKind': typing.Optional[_Anon1],
                'datatype': typing.Optional[IRI],
                'length': typing.Optional[typing.Optional[jsg.Integer]],
                'minlength': typing.Optional[typing.Optional[jsg.Integer]],
                'maxlength': typing.Optional[typing.Optional[jsg.Integer]],
                'pattern': typing.Optional[typing.Optional[jsg.String]],
                'flags': typing.Optional[typing.Optional[jsg.String]],
                'mininclusive': typing.Optional[typing.Optional[jsg.Integer]],
                'minexclusive': typing.Optional[typing.Optional[jsg.Integer]],
                'maxinclusive': typing.Optional[typing.Optional[jsg.Integer]],
                'maxexclusive': typing.Optional[typing.Optional[jsg.Integer]],
                'totaldigits': typing.Optional[typing.Optional[jsg.Integer]],
                'fractiondigits': typing.Optional[typing.Optional[jsg.Integer]]}
    _strict = True

    def __init__(self,
                 nodeKind: typing.Optional[str] = None,
                 datatype: typing.Optional[str] = None,
                 opts_: typing.Union[xsFacet_1_, xsFacet_2_] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.nodeKind = nodeKind
        self.datatype = datatype
        if opts_ is not None:
            if isinstance(opts_, xsFacet_1_):
                if opts_ is not None:
                    if isinstance(opts_, stringFacet_1_):
                        self.length = opts_.length
                        self.minlength = opts_.minlength
                        self.maxlength = opts_.maxlength
                    elif isinstance(opts_, stringFacet_2_):
                        self.pattern = opts_.pattern
                        self.flags = opts_.flags
                    else:
                        raise ValueError(f"Unrecognized value type: {opts_}")
            elif isinstance(opts_, xsFacet_2_):
                self.mininclusive = opts_.mininclusive
                self.minexclusive = opts_.minexclusive
                self.maxinclusive = opts_.maxinclusive
                self.maxexclusive = opts_.maxexclusive
                self.totaldigits = opts_.totaldigits
                self.fractiondigits = opts_.fractiondigits
            else:
                raise ValueError(f"Unrecognized value type: {opts_}")


_CONTEXT.NAMESPACE = locals()



import unittest

from pyjsg.validate_json import JSGPython


class IgnoreTestCase(unittest.TestCase):
    """ Test list of sequences in documentation """
    def test_1(self):
        x = JSGPython('''
.IGNORE target;
doc {a:@string}
''')

        rslt = x.conforms('{"a":"hello", "target":"earthling"}')
        self.assertTrue(rslt.success)


if __name__ == '__main__':
    unittest.main()



import unittest

from jsonasobj.jsonobj import as_json

from pyjsg.jsglib.loader import is_valid, loads
from pyjsg.jsglib import Empty
from pyjsg.validate_json import JSGPython


class JSGNullTestCase(unittest.TestCase):
    def test_required_null(self):
        j = JSGPython('doc { x: @null }', print_python=False)
        rslt = j.conforms('{"x": null}')
        if not rslt.success:
            print(str(rslt))
        self.assertTrue(rslt.success)
        jclass = getattr(j.module, 'doc')
        j2 = jclass()
        self.assertEqual('{}', as_json(j2))
        self.assertFalse(is_valid(j2))
        j2.x = None
        self.assertEqual('{"x": null}', as_json(j2, indent=None))
        self.assertTrue(is_valid(j2))
        j2.x = Empty
        self.assertEqual('{}', as_json(j2))
        self.assertFalse(is_valid(j2))
        with self.assertRaises(ValueError):
            j2.x = 17

    def test_optional_null(self):
        j = JSGPython('doc { x: @null? }', print_python=False)
        rslt = j.conforms('{"x": null}')
        if not rslt.success:
            print(str(rslt))
        self.assertTrue(rslt.success)
        jclass = getattr(j.module, 'doc')
        j2 = jclass()
        self.assertEqual('{}', as_json(j2))
        self.assertTrue(is_valid(j2))
        j2.x = None
        self.assertEqual('{"x": null}', as_json(j2, indent=None))
        self.assertTrue(is_valid(j2))
        j2.x = Empty
        self.assertEqual('{}', as_json(j2))
        self.assertTrue(is_valid(j2))
        with self.assertRaises(ValueError):
            j2.x = 17

    def test_required_any(self):
        j = JSGPython('doc { x: . }', print_python=False)
        rslt = j.conforms('{"x": null}')
        if not rslt.success:
            print(str(rslt))
        self.assertTrue(rslt.success)
        jclass = getattr(j.module, 'doc')
        j2 = jclass()
        self.assertEqual('{}', as_json(j2))
        self.assertFalse(is_valid(j2))
        j2.x = None
        self.assertEqual('{"x": null}', as_json(j2, indent=None))
        self.assertTrue(is_valid(j2))
        j2.x = Empty
        self.assertEqual('{}', as_json(j2))
        self.assertFalse(is_valid(j2))

    def test_various_anys(self):
        j = JSGPython('doc {x: . }')
        for v in ['null', '17', '-22', '-22.0', 'true', '"A string"', '{"x": 243}']:
            json = f'{{"x": {v}}}'
            j2 = loads(json, j.module)
            self.assertEqual(json, as_json(j2, indent=None))
        json = '{"x": -17395e-2}'
        self.assertEqual('{"x": -173.95}', as_json(loads(json, j.module), indent=None))

    def test_various_optional_anys(self):
        j = JSGPython('doc {x: .? }')
        for v in ['null', '17', '-22', '-22.0', 'false', '"A string"', '{"x": null}']:
            json = f'{{"x": {v}}}'
            j2 = loads(json, j.module)
            self.assertEqual(json, as_json(j2, indent=None))
        json = '{"x": -17395e-2}'
        self.assertEqual('{"x": -173.95}', as_json(loads(json, j.module), indent=None))

    def test_optional_any(self):
        j = JSGPython('doc { x: .? }', print_python=False)
        rslt = j.conforms('{"x": null}')
        if not rslt.success:
            print(str(rslt))
        self.assertTrue(rslt.success)
        jclass = getattr(j.module, 'doc')
        j2 = jclass()
        self.assertEqual('{}', as_json(j2))
        self.assertTrue(is_valid(j2))
        j2.x = None
        self.assertEqual('{"x": null}', as_json(j2, indent=None))
        self.assertTrue(is_valid(j2))
        j2.x = Empty
        self.assertEqual('{}', as_json(j2))
        self.assertTrue(is_valid(j2))


if __name__ == '__main__':
    unittest.main()



# Auto generated from JSGPython by PyJSG version 0.8b1
# Generation date: 2018-07-31 12:20
#
import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("doc")


class doc(jsg.JSGObject):
    _reference_types = []
    _members = {'class': jsg.AnyTypeFactory('class', _CONTEXT)}
    _strict = True

    def __init__(self,
                 class_: object = jsg.Empty,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        setattr(self, 'class', class_ if class_ is not jsg.Empty else _kwargs.get('class', jsg.Empty))


_CONTEXT.NAMESPACE = locals()



import unittest

from pyjsg.validate_json import JSGPython


class NotebooksSyntaxTestCase(unittest.TestCase):
    """ Test cases for the notebooks/syntax.ipynb examples"""
    def test_name_valuetype(self):
        x = JSGPython('''doc {
            last_name : @string,       # exactly one last name of type string
            "first name" : @string+      # array or one or more first names
            age : @int?,               # optional age of type int
            weight : @number*,         # array of zero or more weights
            rating : @int{2,},         # at least two ratings
            spin: [@bool{0, 2}]?
        }
        ''', print_python=False)
        r = x.conforms('''
        { "last_name" : "snooter",
          "first name" : ["grunt", "peter"],
          "weight" : [],
          "rating" : [10, 10]
        }''')
        self.assertTrue(r.success, str(r))
        r = x.conforms('''
        { 
          "first name" : ["grunt", "peter"],
          "weight" : [],
          "rating" : [10, 10]
        }''')
        self.assertFalse(r.success)
        self.assertEqual("FAIL - doc: Missing required field: 'last_name'", str(r))
        r = x.conforms('''
        { "last_name" : "snooter",
          "first name" : [],
          "weight" : [],
          "rating" : [9, 9]
        }''')
        self.assertFalse(r.success)
        self.assertEqual(
            "FAIL - Wrong type for first name: [] - expected: <class 'pyjsg.jsglib.jsg_array.first name'> got list",
            str(r))
        r = x.conforms('''
           { "last_name" : "snooter",
             "first name" : "jim",
             "weight" : [],
             "rating" : [1, 2]
           }''')
        self.assertFalse(r.success)
        self.assertEqual(
            "FAIL - Wrong type for first name: 'jim' - expected: <class 'pyjsg.jsglib.jsg_array.first name'> got str",
            str(r))
        r = x.conforms('''
          { "last_name" : "snooter",
            "first name" : [17, 12.3, false],
            "weight" : [],
            "rating" : [1, 1]
          }''')
        self.assertFalse(r.success)
        # TODO: Get these error messages up and running
        print(f"TODO: first name element 0: 17 is not a String\n"
                         f"first name element 1: 12.3 is not a String\n"
                         f"first name element 2: False is not a String !=  {str(r)}")
        # self.assertEqual("FAIL - first name element 0: 17 is not a String\n")
        #                  "first name element 1: 12.3 is not a String\n"
        #                  "first name element 2: False is not a String", str(r))
        r = x.conforms('''
           { "last_name" : "snooter",
             "first name" : ["jim"],
             "weight" : [],
             "rating" : [1]
           }''')
        self.assertFalse(r.success)
        # TODO: Fix this
        print(f"TODO - rating: at least 2 values required - element has 1 != {str(r)}")
        # self.assertEqual("FAIL - rating: at least 2 values required - element has 1", str(r))
        r = x.conforms('''
           { "last_name" : "snooter",
             "first name" : ["jim"],
             "weight" : [],
             "rating" : [1, 10],
             "spin" : [false, true, false]
           }''')
        self.assertFalse(r.success)
        # TODO: Fix this
        print(f"TODO - spin: no more than 2 values permitted - element has 3 != {str(r)}")
        # self.assertEqual("FAIL - spin: no more than 2 values permitted - element has 3", str(r))

    def test_builtins(self):
        x = JSGPython('''
        doc {
            v1: @string,
            v2: @number,
            v3: @int,
            v4: @bool,
            v5: @null,
            v6: @array,
            v7: @object 
        }
        another_object { , }''')

        r = x.conforms('''
        { "v1": "This is text!",
          "v2": -117.432e+2,
          "v3": -100173,
          "v4": false,
          "v5": null,
          "v6": [12, "text", null],
          "v7": {"q": "life", "a": 42}
        }''')
        self.assertTrue(r.success)


if __name__ == '__main__':
    unittest.main()



import unittest

from tests.test_python_generator.utils import PythonGeneratorUtils


class ObjectMapTestCase(PythonGeneratorUtils):
    save_output_files: bool = False

    def test_simple_map(self):
        import tests.test_python_generator.py.objectmap_1 as doc

        jsg = """.TYPE type - Person ;
        Directory {. -> Person}
        Person {name: @string age: @int?}
        """
        test_cases = [
            """{"type": "Directory",
                "1234": {"name": "Sam Smith", "age": 43},
                "1245": {"name": "Jill Jones", "age": 17},
                "Unk person": {"name": "Destroyer"}}""",
            '{"type": "Directory"}'
        ]

        fail_cases = [
            """{"type": "Directory",
                "1234": {"age": 43}}""",
            """{"type": "Directory",
                "v": 1}""",
            """{"type": "Directory",
                "v": {"g": null}"""
        ]
        self.do_test(jsg, 'objectmap_1', doc, test_cases, {}, fail_cases, print_python=False)

    def test_cardinality(self):
        import tests.test_python_generator.py.objectmap_2 as doc

        jsg = """Directory {PNAME -> Person{2,3}}
        Person {name: @string age: @int?}

        @terminals
        PNAME: [A-Z][0-9]+ ;        
        """
        test_cases = [
            """{"A1234": {"name": "Sam Smith", "age": 43},
                "A1245": {"name": "Jill Jones", "age": 17}}""",
            """{"A1234": {"name": "Sam Smith", "age": 43},
                "A1245": {"name": "Jill Jones", "age": 17},
                "A12345": {"name": "Destroyer"}}""",
        ]
        fail_cases = ["""{"B0": {"name": "Sam Smith", "age": 43},
                "B1": {"name": "Jill Jones", "age": 17},
                "B2": {"name": "Destroyer"}},
                "B3"" {"name": "toomany"}}"""
        ]
        self.do_test(jsg, 'objectmap_2', doc, test_cases, {}, fail_cases, print_python=False)

    def test_key_types_2(self):
        import tests.test_python_generator.py.objectmap_3 as doc

        jsg = """Directory {PNAME -> Person}
Person {name: @string age: @int?}

@terminals
PNAME: [A-Z][0-9]+ ;        
"""
        test_cases = [
            '{"A1": {"name": "Sam Smith", "age": 43}}',
            '{"B17": {"name": "Jill Jones", "age": 17}}',
            '{}'
        ]
        fail_cases = [

        ]
        self.do_test(jsg, 'objectmap_3', doc, test_cases, {}, fail_cases, print_python=False)


if __name__ == '__main__':
    unittest.main()



import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE = "type"
_CONTEXT.TYPE_EXCEPTIONS.append("person")


class _Anon1(jsg.JSGString):
    pattern = jsg.JSGPattern(r'(m)|(f)')


class details(jsg.JSGObject):
    _reference_types = []
    _members = {'id': jsg.String}
    _strict = False

    def __init__(self,
                 id: str = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.id = id



class person(jsg.JSGObject):
    _reference_types = [details]
    _members = {'name': jsg.String,
                'gender': _Anon1,
                'active': jsg.Boolean,
                'id': jsg.ArrayFactory('id', _CONTEXT, jsg.String, 0, None)}
    _strict = True

    def __init__(self,
                 name: str = None,
                 gender: str = None,
                 active: bool = None,
                 id: str = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.name = name
        self.gender = gender
        self.active = active
        self.id = id



class company(jsg.JSGObject):
    _reference_types = []
    _members = {'name': jsg.String,
                'year founded': typing.Optional[jsg.Integer],
                'employees': jsg.ArrayFactory('employees', _CONTEXT, person, 2, None)}
    _strict = True

    def __init__(self,
                 name: str = None,
                 employees: typing.List[person] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.name = name
        setattr(self, 'year founded', _kwargs.get('year founded', None))
        self.employees = employees


_CONTEXT.NAMESPACE = locals()



import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("stringFacet")
_CONTEXT.TYPE_EXCEPTIONS.append("numericFacet")
_CONTEXT.TYPE_EXCEPTIONS.append("xsFacet")
_CONTEXT.TYPE_EXCEPTIONS.append("labeledNodeConstraint")


class INTEGER(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[0-9]*')


class DECIMAL(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[0-9]*')


class DOUBLE(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[0-9]*')


class STRING(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[A-Z]*')


class stringFacet(jsg.JSGObject):
    _reference_types = []
    _members = {'length': typing.Optional[INTEGER],
                'minlength': typing.Optional[INTEGER],
                'maxlength': typing.Optional[INTEGER],
                'pattern': STRING,
                'flags': typing.Optional[STRING]}
    _strict = True

    def __init__(self,
                 length: typing.Optional[str] = None,
                 minlength: typing.Optional[str] = None,
                 maxlength: typing.Optional[str] = None,
                 pattern: str = None,
                 flags: typing.Optional[str] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.length = length
        self.minlength = minlength
        self.maxlength = maxlength
        self.pattern = pattern
        self.flags = flags



class numericFacet(jsg.JSGObject):
    _reference_types = []
    _members = {'mininclusive': typing.Optional[DOUBLE],
                'minexclusive': typing.Optional[DOUBLE],
                'maxinclusive': typing.Optional[DOUBLE],
                'maxexclusive': typing.Optional[DOUBLE],
                'totaldigits': typing.Optional[INTEGER],
                'fractiondigits': typing.Optional[INTEGER]}
    _strict = True

    def __init__(self,
                 mininclusive: typing.Optional[str] = None,
                 minexclusive: typing.Optional[str] = None,
                 maxinclusive: typing.Optional[str] = None,
                 maxexclusive: typing.Optional[str] = None,
                 totaldigits: typing.Optional[str] = None,
                 fractiondigits: typing.Optional[str] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.mininclusive = mininclusive
        self.minexclusive = minexclusive
        self.maxinclusive = maxinclusive
        self.maxexclusive = maxexclusive
        self.totaldigits = totaldigits
        self.fractiondigits = fractiondigits



class xsFacet(jsg.JSGObject):
    _reference_types = [stringFacet, numericFacet]
    _members = {'length': typing.Optional[INTEGER],
                'minlength': typing.Optional[INTEGER],
                'maxlength': typing.Optional[INTEGER],
                'pattern': STRING,
                'flags': typing.Optional[STRING],
                'mininclusive': typing.Optional[DOUBLE],
                'minexclusive': typing.Optional[DOUBLE],
                'maxinclusive': typing.Optional[DOUBLE],
                'maxexclusive': typing.Optional[DOUBLE],
                'totaldigits': typing.Optional[INTEGER],
                'fractiondigits': typing.Optional[INTEGER]}
    _strict = True

    def __init__(self,
                 length: typing.Optional[str] = None,
                 minlength: typing.Optional[str] = None,
                 maxlength: typing.Optional[str] = None,
                 pattern: str = None,
                 flags: typing.Optional[str] = None,
                 mininclusive: typing.Optional[str] = None,
                 minexclusive: typing.Optional[str] = None,
                 maxinclusive: typing.Optional[str] = None,
                 maxexclusive: typing.Optional[str] = None,
                 totaldigits: typing.Optional[str] = None,
                 fractiondigits: typing.Optional[str] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.length = length
        self.minlength = minlength
        self.maxlength = maxlength
        self.pattern = pattern
        self.flags = flags
        self.mininclusive = mininclusive
        self.minexclusive = minexclusive
        self.maxinclusive = maxinclusive
        self.maxexclusive = maxexclusive
        self.totaldigits = totaldigits
        self.fractiondigits = fractiondigits



class labeledNodeConstraint(jsg.JSGObject):
    _reference_types = [xsFacet]
    _members = {'first': typing.Optional[INTEGER],
                'length': jsg.ArrayFactory('length', _CONTEXT, typing.Optional[INTEGER], 0, None),
                'minlength': jsg.ArrayFactory('minlength', _CONTEXT, typing.Optional[INTEGER], 0, None),
                'maxlength': jsg.ArrayFactory('maxlength', _CONTEXT, typing.Optional[INTEGER], 0, None),
                'pattern': jsg.ArrayFactory('pattern', _CONTEXT, STRING, 0, None),
                'flags': jsg.ArrayFactory('flags', _CONTEXT, typing.Optional[STRING], 0, None),
                'mininclusive': jsg.ArrayFactory('mininclusive', _CONTEXT, typing.Optional[DOUBLE], 0, None),
                'minexclusive': jsg.ArrayFactory('minexclusive', _CONTEXT, typing.Optional[DOUBLE], 0, None),
                'maxinclusive': jsg.ArrayFactory('maxinclusive', _CONTEXT, typing.Optional[DOUBLE], 0, None),
                'maxexclusive': jsg.ArrayFactory('maxexclusive', _CONTEXT, typing.Optional[DOUBLE], 0, None),
                'totaldigits': jsg.ArrayFactory('totaldigits', _CONTEXT, typing.Optional[INTEGER], 0, None),
                'fractiondigits': jsg.ArrayFactory('fractiondigits', _CONTEXT, typing.Optional[INTEGER], 0, None),
                'last': jsg.ArrayFactory('last', _CONTEXT, STRING, 1, None)}
    _strict = True

    def __init__(self,
                 first: typing.Optional[str] = None,
                 length: typing.Optional[str] = None,
                 minlength: typing.Optional[str] = None,
                 maxlength: typing.Optional[str] = None,
                 pattern: str = None,
                 flags: typing.Optional[str] = None,
                 mininclusive: typing.Optional[str] = None,
                 minexclusive: typing.Optional[str] = None,
                 maxinclusive: typing.Optional[str] = None,
                 maxexclusive: typing.Optional[str] = None,
                 totaldigits: typing.Optional[str] = None,
                 fractiondigits: typing.Optional[str] = None,
                 last: typing.List[str] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.first = first
        self.length = length
        self.minlength = minlength
        self.maxlength = maxlength
        self.pattern = pattern
        self.flags = flags
        self.mininclusive = mininclusive
        self.minexclusive = minexclusive
        self.maxinclusive = maxinclusive
        self.maxexclusive = maxexclusive
        self.totaldigits = totaldigits
        self.fractiondigits = fractiondigits
        self.last = last


_CONTEXT.NAMESPACE = locals()




from pyjsg.jsglib import JSGString, JSGPattern


class HEX(JSGString):
    pattern = JSGPattern(r'[0-9]|[A-F]|[a-f]')

class PN_CHARS_BASE(JSGString):
    pattern = JSGPattern(r'[A-Z]|[a-z]|[\u00C0-\u00D6]|[\u00D8-\u00F6]|[\u00F8-\u02FF]|[\u0370-\u037D]|[\u037F-\u1FFF]|[\u200C-\u200D]|[\u2070-\u218F]|[\u2C00-\u2FEF]|[\u3001-\uD7FF]|[\uF900-\uFDCF]|[\uFDF0-\uFFFD]|[\u10000-\uEFFFF]')


class PN_CHARS_U(JSGString):
    pattern = JSGPattern(r'{PN_CHARS_BASE}|_'.format(PN_CHARS_BASE=PN_CHARS_BASE.pattern))


class UCHAR(JSGString):
    pattern = JSGPattern(r'\\\\u{HEX}{HEX}{HEX}{HEX}|\\\\U{HEX}{HEX}{HEX}{HEX}{HEX}{HEX}{HEX}{HEX}'.format(HEX=HEX.pattern))


class PN_CHARS(JSGString):
    pattern = JSGPattern(r'{PN_CHARS_U}|\-|[0-9]|\\u00B7|[\u0300-\u036F]|[\u203F-\u2040]'.format(PN_CHARS_U=PN_CHARS_U.pattern))


class IRI(JSGString):
    pattern = JSGPattern(r'({PN_CHARS}|\.|\:|\/|\\\\|\#|\@|\%|\&|{UCHAR})*'.format(PN_CHARS=PN_CHARS.pattern, UCHAR=UCHAR.pattern))



import unittest

from tests.test_python_generator.utils import PythonGeneratorUtils


class CardinalityTestCase(PythonGeneratorUtils):
    save_output_files: bool = False

    def test_list_cardinalities(self):
        import tests.test_python_generator.py.cardinality_1 as doc

        jsg = '''
        doc {opt: .?
          req: .
          l0n: .*
          l1n: .+
          l01: [.?]
          l11: [.{1}]
          l0na: [.*]
          l1na: [.+]
          optl0n: [.*]?
          optl1n: [.+]?
        }'''
        test_cases = []
        self.do_test(jsg, 'cardinality_1', doc, test_cases, {})

    def test_list_cardinalities_2(self):
        import tests.test_python_generator.py.cardinality_2 as doc

        jsg = '''
           doc {opt: @string?
             req: @string
             l0n: @string*
             l1n: @string+
             l01: [@string?]
             l11: [@string]
             l0na: [@string*]
             l1na: [@string+]
             optl0n: [@string*]?
             optl1n: [@string+]?
           }'''
        test_cases = []
        self.do_test(jsg, 'cardinality_2', doc, test_cases, {})

    def test_nested_lists(self):
        import tests.test_python_generator.py.cardinality_3 as doc

        jsg = '''
        doc {l0nl0n: [.*]*} '''
        test_cases = []
        self.do_test(jsg, 'cardinality_3', doc, test_cases, {})

    def test_nested_cardinalities(self):
        import tests.test_python_generator.py.cardinality_4 as doc
        # TODO: Need to be able to pass a constructor template so that we can have an either 2 or 3 arrays
        # each consisting of an array of 2 to 8 strings.  At the moment, the inner cardinality is lost
        # Probably the correct solution is to create an anonymous inner constructor?
        jsg = 'doc {"l23 38n": [@string{2,8}]{2, 3}}'
        test_cases = []
        self.do_test(jsg, 'cardinality_4', doc, test_cases, {})


if __name__ == '__main__':
    unittest.main()



# Auto generated from JSGPython by PyJSG version 0.8b1
# Generation date: 2018-07-31 11:31
#
import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("doc")
_CONTEXT.TYPE_EXCEPTIONS.append("another_object")


class doc(jsg.JSGObject):
    _reference_types = []
    _members = {'class': jsg.String,
                'def': jsg.Number,
                'import': jsg.Integer,
                'with': jsg.Boolean,
                'if': jsg.JSGNull,
                'else': jsg.ArrayFactory('else', _CONTEXT, jsg.AnyType, 0, None),
                'raise': jsg.ObjectFactory('raise', _CONTEXT, jsg.Object)}
    _strict = True

    def __init__(self,
                 class_: str = None,
                 def_: float = None,
                 import_: int = None,
                 with_: bool = None,
                 if_: type(None) = jsg.Empty,
                 else_: list = None,
                 raise_: object = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        setattr(self, 'class', class_ if class_ is not None else _kwargs.get('class', None))
        setattr(self, 'def', def_ if def_ is not None else _kwargs.get('def', None))
        setattr(self, 'import', import_ if import_ is not None else _kwargs.get('import', None))
        setattr(self, 'with', with_ if with_ is not None else _kwargs.get('with', None))
        setattr(self, 'if', if_ if if_ is not jsg.Empty else _kwargs.get('if', jsg.Empty))
        setattr(self, 'else', else_ if else_ is not None else _kwargs.get('else', None))
        setattr(self, 'raise', raise_ if raise_ is not None else _kwargs.get('raise', None))



class another_object(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = False

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)


_CONTEXT.NAMESPACE = locals()



import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("doc")


class _Anon1(jsg.JSGString):
    pattern = jsg.JSGPattern(r'ready')


class doc(jsg.JSGObject):
    _reference_types = []
    _members = {'status': _Anon1}
    _strict = True

    def __init__(self,
                 status: str = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.status = status


_CONTEXT.NAMESPACE = locals()




import unittest
from typing import Dict, Optional

from dict_compare import dict_compare
from io import StringIO
from jsonasobj import loads as json_loads

from pyjsg.jsglib import JSGNull, String, Integer, Boolean, Number, JSGContext, JSGObject, Empty

_CONTEXT = JSGContext()


class ObjectTestCase(unittest.TestCase):
    def check_json(self, t: JSGObject, c: str):
        self.assertTrue(dict_compare(json_loads(c)._as_dict, t._as_json_obj()))

    def test_simple_object(self):
        class Person(JSGObject):
            _reference_types = []
            _members = {'name': String,
                        'age': Integer,
                        'married': Boolean,
                        'weight': Number,
                        'tag': Optional[JSGNull]}
            _strict = True

            def __init__(self,
                         name: str = None,
                         age: int = None,
                         married: bool = None,
                         weight: float = None,
                         tag: Optional[type(None)] = Empty,
                         **_kwargs: Dict[str, object]):
                super().__init__(_CONTEXT, **_kwargs)
                self.name = name
                self.age = age
                self.married = married
                self.weight = weight
                self.tag = tag

        log = StringIO()

        x = Person()
        x.name = "Grunt P Snooter"
        x = Person(name="Sally Pope", age=42, married=True, weight=117)
        x.name = "Joe"
        self.check_json(x, '{"name": "Sally Pope", "age": 42, "married": true, "weight": 117.5}')
        x = Person("Sally Pope", "42")
        self.check_json(x, '{"name": "Sally Pope", "age": 42}')
        x = Person("Sally Pope")
        self.assertFalse(x._is_valid(log))
        self.assertEqual('Person: Missing required field: \'age\'\n'
                         'Person: Missing required field: \'married\'\n'
                         'Person: Missing required field: \'weight\'\n', log.getvalue())
        log = StringIO()
        x.age = 99
        self.assertFalse(x._is_valid())
        x.married = False
        self.assertFalse(x._is_valid())
        x.weight = "112"
        self.assertTrue(x._is_valid())
        self.check_json(x, '{"name": "Sally Pope", "age": 99, "married": false, "weight": 112}')
        x.tag = None
        self.check_json(x, '{"name": "Sally Pope", "age": 99, "married": false, "weight": 112, "tag": null}')
        x.tag = Empty
        self.check_json(x, '{"name": "Sally Pope", "age": 99, "married": false, "weight": 112}')
        with self.assertRaises(ValueError):
            x.age = "abc"
        del x.weight
        x._is_valid(log)
        self.assertEqual('Person: Missing required field: \'weight\'\n', log.getvalue())


if __name__ == '__main__':
    unittest.main()



import os
import re
import unittest
from typing import List, Dict, Any

from pyjsg.jsglib.loader import loads, is_valid, logger
from pyjsg.validate_json import JSGPython


class PythonGeneratorUtils(unittest.TestCase):
    cwd = os.path.abspath(os.path.join(os.path.dirname(__file__), 'py'))
    save_output_files: bool = False
    save_all_output_files: bool = False

    def tearDown(self):
        self.assertFalse(self.save_output_files, "save_output_files is True")
        if self.save_all_output_files:
            print("Warning: tests.test_python_generator.utils save_all_output_files is True")

    @staticmethod
    def _strip_details(txt: str) -> str:
        txt = re.sub(r'(# Auto generated from JSGPython by PyJSG version ).*', r'\1', txt)
        return re.sub(r'(# Generation date:).*', r'\1', txt).strip()

    def do_test(self, jsg: str, test_file: str, module, passing_json: List[str], failing_vars: Dict[str, Any],
                failing_json: List[str] = None, print_python: bool = False) -> None:
        x = JSGPython(jsg, print_python=print_python)
        py_file = os.path.join(self.cwd, test_file + '.py')
        with open(py_file) as f:
            expected = self._strip_details(f.read())
        actual = self._strip_details(x.python)
        if expected != actual:
            if self.save_output_files or self.save_all_output_files:
                with open(py_file, 'w') as f:
                    f.write(x.python)
                    print(f"***** {py_file} updated *****")
            self.maxDiff = None
            self.assertEqual(expected, actual)

        for p in passing_json:
            json_doc = loads(p, module)
            log = logger()
            doc_is_valid = is_valid(json_doc, log)
            self.assertTrue(doc_is_valid, log.getvalue())

        for f in failing_json if failing_json else []:
            doc_is_valid = True
            json_doc = None
            error = ""
            try:
                json_doc = loads(f, module)
            except ValueError as e:
                doc_is_valid = False
                error = e
            if doc_is_valid:
                log = logger()
                doc_is_valid = is_valid(json_doc, log)
                error = log.getvalue()
            self.assertFalse(doc_is_valid, f)

        for k, v in failing_vars.items():
            with self.assertRaises(ValueError):
                vv = f'"{v}"' if isinstance(v, str) else str(v)
                loads(f'{{"{k}": {vv}}}', module)



import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("EachOf")
_CONTEXT.TYPE_EXCEPTIONS.append("TripleConstraint")
_CONTEXT.TYPE_EXCEPTIONS.append("OneOf")


class _Anon1(jsg.JSGString):
    pattern = jsg.JSGPattern(r'unbounded')


class IRI(jsg.String):
    pattern = jsg.JSGPattern(r'')


class BNODE(jsg.String):
    pattern = jsg.JSGPattern(r'')


class INTEGER(jsg.String):
    pattern = jsg.JSGPattern(r'')


class EachOf(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class TripleConstraint(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



tripleExprLabel = typing.Union[IRI, BNODE]


tripleExpr = typing.Union[EachOf, "OneOf", typing.Union[IRI, BNODE]]


class OneOf(jsg.JSGObject):
    _reference_types = []
    _members = {'id': typing.Optional[tripleExprLabel],
                'expressions': jsg.ArrayFactory('expressions', _CONTEXT, typing.Union[EachOf, "OneOf", typing.Union[IRI, BNODE]], 2, None),
                'min': typing.Optional[INTEGER],
                'max': typing.Optional[typing.Union[_Anon1, INTEGER]]}
    _strict = True

    def __init__(self,
                 id: typing.Optional[typing.Union[str, str]] = None,
                 expressions: typing.List[typing.Union[EachOf, "OneOf", typing.Union[str, str]]] = None,
                 min: typing.Optional[str] = None,
                 max: typing.Optional[typing.Union[str, str]] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.id = id
        self.expressions = expressions
        self.min = min
        self.max = max


_CONTEXT.NAMESPACE = locals()



import unittest

from tests.test_python_generator.utils import PythonGeneratorUtils


class AnyTypeTestCase(PythonGeneratorUtils):
    save_output_files: bool = False

    def test_any_object(self):
        import tests.test_python_generator.py.any_type as doc

        jsg = 'doc {status: .}'
        test_cases = [
            '{"status": 17}',
            '{"status": "test"}',
            '{"status": null}',
            '{"status": []}',
            '{"status": {"status": -117.2}}',
            '{"status": [{"status": "inside"}]}'
        ]
        self.do_test(jsg, 'any_type', doc, test_cases, {})

        import tests.test_python_generator.py.any_type_2 as doc2
        jsg2 = 'doc {class: .}'
        test_cases2 = [
            '{"class": 17}',
            '{"class": "test"}',
            '{"class": null}',
            '{"class": []}',
            '{"class": {"class": -117.2}}',
            '{"class": [{"class": "inside"}]}'
        ]
        self.do_test(jsg2, 'any_type_2', doc2, test_cases2, {})

        import tests.test_python_generator.py.any_type_3 as doc3
        jsg3 = 'doc {"A 1": .}'
        test_cases3 = [
            '{"A 1": 17}',
            '{"A 1": "test"}',
            '{"A 1": null}',
            '{"A 1": []}',
            '{"A 1": {"A 1": -117.2}}',
            '{"A 1": [{"A 1": "inside"}]}'
        ]
        self.do_test(jsg3, 'any_type_3', doc3, test_cases3, {})

if __name__ == '__main__':
    unittest.main()



# Auto generated from JSGPython by PyJSG version 0.8b1
# Generation date: 2018-07-31 11:31
#
import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("doc")
_CONTEXT.TYPE_EXCEPTIONS.append("another_object")


class doc(jsg.JSGObject):
    _reference_types = []
    _members = {'v 1': jsg.String,
                'v 2': jsg.Number,
                'v 3': jsg.Integer,
                'v 4': jsg.Boolean,
                'v 5': jsg.JSGNull,
                'v 6': jsg.ArrayFactory('v 6', _CONTEXT, jsg.AnyType, 0, None),
                'v 7': jsg.ObjectFactory('v 7', _CONTEXT, jsg.Object)}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        setattr(self, 'v 1', _kwargs.get('v 1', None))
        setattr(self, 'v 2', _kwargs.get('v 2', None))
        setattr(self, 'v 3', _kwargs.get('v 3', None))
        setattr(self, 'v 4', _kwargs.get('v 4', None))
        setattr(self, 'v 5', _kwargs.get('v 5', jsg.Empty))
        setattr(self, 'v 6', _kwargs.get('v 6', None))
        setattr(self, 'v 7', _kwargs.get('v 7', None))



class another_object(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = False

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)


_CONTEXT.NAMESPACE = locals()




import unittest

from pyjsg.jsglib import JSGString, JSGPattern, Boolean


class Issue7Test(unittest.TestCase):
    def test_issue_7(self):
        class LANGTAG(JSGString):
            pattern = JSGPattern(r'[a-zA-Z]+(\-([a-zA-Z0-9])+)*')

        x = LANGTAG("de")
        self.assertNotEqual("fr", x)

    def test_issue_7_bool(self):
        # def __setattr__(self, key, value):
        #     if key == "val" and value is not None:
        #         self.__dict__[key] = value if isinstance(value, bool) else Boolean.true_pattern.matches(str(value))
        #     else:
        #         self.__dict__[key] = value

        # Update: Bool simply can't be wrapped -- skip the val stuff
        b1 = Boolean("True")
        with self.assertRaises(ValueError):
            Boolean("Aardvark")
        b2 = Boolean("False")
        b1 = b2
        self.assertFalse(b1)
        b1 = Boolean("False")
        b1 = "True"
        b1 = "False"
        with self.assertRaises(ValueError):
            b1 = Boolean(0)


if __name__ == '__main__':
    unittest.main()



import unittest

from pyjsg.validate_json import JSGPython


class GeneSeqTestCase(unittest.TestCase):
    """ Test list of sequences in documentation """
    def test_1(self):
        x = JSGPython('''
doc {
    sequences: [(RNASEQ|DNASEQ)]
}

@terminals
RNASEQ: [ACGU]+ ;
DNASEQ: [ACGT]+ ;
''')

        rslt = x.conforms('''
{ "sequences": [
    "GCUACGGAGCUUGGAGCUAG",
    "ATTTTGCGAGGTCCC"
   ]
}''')
        self.assertTrue(rslt.success)


if __name__ == '__main__':
    unittest.main()



import sys
import unittest

from tests.test_python_generator.utils import PythonGeneratorUtils


class MacroTestCase(PythonGeneratorUtils):
    save_output_files: bool = False

    def test_macro1(self):
        import tests.test_python_generator.py.macrotest_1 as doc

        jsg = 'Shape {id:(@int|@string)?}'
        test_cases = [
            '{"id": 17}',
            '{"id": "abc"}'
        ]
        fail_cases = [
            '{"id": 12.2}',
            '{"id": 17, "j": 1}'
        ]
        self.do_test(jsg, 'macrotest_1', doc, test_cases, {}, fail_cases)

    def test_macro2(self):
        import tests.test_python_generator.py.macrotest_2 as doc2
        jsg2 = """
shapeExprLabel   = @int|@string ;
Shape            {id:shapeExprLabel?}
"""
        test_cases = [
            '{"id": 17}',
            '{"id": "abc"}'
        ]
        fail_cases = [
            '{"id": 12.2}',
            '{"id": 17, "j": 1}'
        ]
        self.do_test(jsg2, 'macrotest_2', doc2, test_cases, {}, fail_cases)

    def test_macro3(self):
        # Subtle change in earlier python - ':' is escaped in regular expression strings
        if sys.version_info < (3, 7):
            import tests.test_python_generator.py.macrotest_3_36 as doc3
            testfile = "macrotest_3_36"
        else:
            import tests.test_python_generator.py.macrotest_3 as doc3
            testfile = "macrotest_3"

        jsg3 = """
shapeExprLabel   = IRIREF|BNODE ;
Shape            {id:shapeExprLabel?}

@terminals
IRIREF           : [A-Z]+ ;
BNODE            : '_:' [A-Z]+ ;\
"""
        test_cases = [
            '{"id": "AAA"}',
            '{"id": "_:AAA"}'
        ]
        fail_cases = [
            '{"id": "aaa"}',
            '{"id": "AAA", "j": 1}'
        ]
        self.do_test(jsg3, testfile, doc3, test_cases, {}, fail_cases, print_python=False)


if __name__ == '__main__':
    unittest.main()



import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("shapeExprLabel")
_CONTEXT.TYPE_EXCEPTIONS.append("labeledShapeOr")


class _Anon1(jsg.JSGString):
    pattern = jsg.JSGPattern(r'ShapeOr')


class SHAPEEXPR(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[a-z]')


class shapeExprLabel(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class labeledShapeOr(jsg.JSGObject):
    _reference_types = []
    _members = {'type': _Anon1,
                'id': shapeExprLabel,
                'shapeExprs': jsg.ArrayFactory('shapeExprs', _CONTEXT, SHAPEEXPR, 2, None)}
    _strict = True

    def __init__(self,
                 type: str = None,
                 id: shapeExprLabel = None,
                 shapeExprs: typing.List[str] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.type = type
        self.id = id
        self.shapeExprs = shapeExprs


_CONTEXT.NAMESPACE = locals()




import os
import unittest
from shutil import copyfile

from pyjsg.validate_json import JSGPython

json_str = """{
  "@context": "http://www.w3.org/ns/shex.jsonld",
  "type": "Schema",
  "shapes": [
    {
      "id": "http://a.example/S1",
      "type": "Shape"
    }
  ]
}"""


class NamespaceTestIssue(unittest.TestCase):
    def test_namespace(self):
        """ There is something pretty subtle about caching and loading ShExJ.py in two different namespaces. The
        test_jsg_files.py test includes a validate function, which dynamically loads all of the generated python
        modules in tests/data/py.  Pretty much no matter how we do it, the module for that dynamic load isn't going
        to be the same as what you get from a straight import.

        We've tried clearing the cache, giving each python file a different name, setting the _ForwardRef
        __forward_evaluated__ to false, but nothing seems to work.  Long and short of it is, you shouldn't
        run test_shexj.py in the same thread as test_jsg_files.

        Update: we removed the _ForwardRef entries and switched to the internal forward typing (x:"Foo") if class Foo
        is further down in the class.  We add a NAMESPACE to the context at the very end of the file and pass it
        through to all of the resolution functions.  BIG HIGH FIVE on this one!
        """
        from pyjsg.parser_impl.generate_python import evaluate

        # # First load done from the testing namespace
        pyfile = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'test_basics', 'py', 'ShExJ.py'))
        evaluate("test_namespace", pyfile, False)
        result = JSGPython(None, pyfile).conforms(json_str)
        self.assertTrue(result.success)

        # Second load from a different file
        pyfile2 = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'test_jsglib', 'py', 'ShExJ.py'))
        copyfile(pyfile, pyfile2)
        result = JSGPython(None, pyfile2).conforms(json_str)
        if not result.success:
            print(result.fail_reason)

        self.assertTrue(result.success)


if __name__ == '__main__':
    unittest.main()



import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE = "type"
_CONTEXT.TYPE_EXCEPTIONS.append("ObjectLiteral")
_CONTEXT.TYPE_EXCEPTIONS.append("labeledShapeOr")
_CONTEXT.TYPE_EXCEPTIONS.append("labeledShapeAnd")
_CONTEXT.TYPE_EXCEPTIONS.append("labeledShapeNot")
_CONTEXT.TYPE_EXCEPTIONS.append("labeledNodeConstraint")
_CONTEXT.TYPE_EXCEPTIONS.append("labeledShape")
_CONTEXT.TYPE_EXCEPTIONS.append("labeledShapeExternal")
_CONTEXT.IGNORE.append("This")
_CONTEXT.IGNORE.append("That")

_CONTEXT.NAMESPACE = locals()
